'''
@file: DSSP.py

@author: Jaume Bonet
@mail:   jaume.bonet@gmail.com
@date:   03/2013

@ [oliva's lab](http://sbi.imim.es)

@class: DSSP
'''
from SBI.data import aminoacids3to1     as a3to1
from SBI.data import aminoacids_surface as asurf


class DSSP(object):
    '''
    Stores a DSSP prediction for a single amino acid.

    '''
    _EXPOSED_THRESHOLD = 2.5
    # Why? DSSP exposition refers only to secondary chain (aprox.) see PMID: 15768403

    def __init__(self, secondary_structure, accessibility, amino):
        '''
        @param:    secondary_structure
        @pdef:     1 letter code of secondary structure
        @ptype:    {String}

        @param:    accessibility
        @pdef:     numeric value of accessibility of the residue
        @ptype:    {Integer}

        @param:    amino
        @pdef:     aminoacid name (1 or 3 letter code)
        @ptype:    {String}

        '''
        self._ss         = secondary_structure
        self._access     = int(accessibility)
        self._type       = amino if len(amino) == 1 else a3to1[amino]
        if amino in asurf:
            self._access10   = int((10 * self._access) / asurf[amino])
            self._accesscode = DSSP._codifyaccess((float(self._access) / asurf[amino]) * 100)
        else:
            self._access10   = 1
            self._accesscode = 1
        self._exposed    = self._access10 > DSSP.exposition_threshold
        self._rnhoa      = None
        self._enhoa      = None
        self._rohna      = None
        self._eohna      = None
        self._rnhob      = None
        self._enhob      = None
        self._rohnb      = None
        self._eohnb      = None

    ##############
    # ATTRIBUTES #
    ##############
    @property
    def secondary_structure(self):
        '''
        @return: {String}
        '''
        return self._ss

    @property
    def aminoacid(self):
        '''
        @return: {String}
        '''
        return self._type

    @property
    def accessibility(self):
        '''
        @return: {Integer}
        '''
        return self._access

    @property
    def accessibility10(self):
        '''
        @return: {Integer}
        '''
        return self._access10

    @property
    def accesscode(self):
        '''
        @return: {String}
        '''
        return str(self._accesscode)

    @staticmethod
    @property
    def exposition_threshold():
        '''
        @return: {Float}
        '''
        return DSSP._EXPOSED_THRESHOLD

    @property
    def exposed(self):
        '''
        @return: {Boolean}
        '''
        return self._exposed

    @property
    def nhoa(self):
        '''
        @return: {List} of {Integer} and {Float}
        '''
        return self._rnhoa, self._enhoa

    @property
    def ohna(self):
        '''
        @return: {List} of {Integer} and {Float}
        '''
        return self._rohna, self._eohna

    @property
    def nhob(self):
        '''
        @return: {List} of {Integer} and {Float}
        '''
        return self._rnhob, self._enhob

    @property
    def ohnb(self):
        '''
        @return: {List} of {Integer} and {Float}
        '''
        return self._rohnb, self._eohnb

    ###########
    # METHODS #
    ###########
    def add_hydrogen_links(self, nhoa, ohna, nhob, ohnb):
        '''
        Hydrogen link data from the DSSP file

        @param:    nhoa
        @pdef:     N-HO link CA
        @ptype:    {List}

        @param:    ohna
        @pdef:     OH-N link CA
        @ptype:    {List}

        @param:    nhob
        @pdef:     N-HO link CB
        @ptype:    {List}

        @param:    ohnb
        @pdef:     OH-N link CB
        @ptype:    {List}
        '''
        self._rnhoa = int(nhoa.split(',')[0].strip())
        self._enhoa = float(nhoa.split(',')[1].strip())
        self._rohna = int(ohna.split(',')[0].strip())
        self._eohna = float(ohna.split(',')[1].strip())
        self._rnhob = int(nhob.split(',')[0].strip())
        self._enhob = float(nhob.split(',')[1].strip())
        self._rohnb = int(ohnb.split(',')[0].strip())
        self._eohnb = float(ohnb.split(',')[1].strip())

    ###################
    # PRIVATE METHODS #
    ###################
    @staticmethod
    def _codifyaccess(value):
        '''
        Codifies the access value into a numeric code.

        @param:    value
        @pdef:     actual accessibility
        @ptype:    {Float}

        @return: {String}
        '''
        value = float(value)
        if value == 0:
            return '*'  # Buried
        elif value > 100:
            return '?'  # PTM that make the Aa bigger
        else:
            if value > 0  and value <= 10:
                return '1'
            elif value > 10 and value <= 20:
                return '2'
            elif value > 20 and value <= 30:
                return '3'
            elif value > 30 and value <= 40:
                return '4'
            elif value > 40 and value <= 50:
                return '5'
            elif value > 50 and value <= 60:
                return '6'
            elif value > 60 and value <= 70:
                return '7'
            elif value > 70 and value <= 80:
                return '8'
            elif value > 80 and value <= 90:
                return '9'
            elif value > 90 and value <= 100:
                return '#'



'''
@file: dblink.py

@author: Jaume Bonet
@mail:   jaume.bonet@gmail.com
@date:   2014

@ [oliva's lab](http://sbi.imim.es)

@class: DBlink
'''
from abc import ABCMeta, abstractmethod
from datetime import date
# import time
import urllib

import os
import json

from SBI.beans import Path
from SBI.beans import File
from SBI       import SBIglobals as SBIg


class DBlink(object):
    '''
    Manages the connection to databases.
    It creates a local copy of the DB that can be queried a posteriori.
    '''
    __metaclass__ = ABCMeta

    _MANDATORY_FILES = []
    _ITEM_FILES      = []
    _SOURCES         = []
    _SOURCES_SIZES   = []
    _SHOW_LINK       = None

    _DBOBJECT        = None

    _CONTROL_FILE    = 'release.json.gz'
    _TODAY           = date.today().isoformat()
    _RELEASE         = {'db':            None,
                        'date':          _TODAY,
                        'total_items':   {},
                        'new_items':     {},
                        'update_items':  {},
                        'deleted_items': {}}

    def __init__(self, local):
        '''
        @param:    local
        @pdef:     local directory to store the database. Create if not exist
        @ptype:    {String}
        '''
        self._local = local
        Path.mkdir(self._local)

    ##############
    # ATTRIBUTES #
    ##############
    @property
    def local(self):
        '''
        Directory of the local database

        @return: {String}
        '''
        return os.path.abspath(self._local)

    @property
    def release(self):
        '''
        Retrieves release data for the database.
        Not according to the DB release, but to when we downloaded it.

        @returns: {Dictionary}
        '''
        if os.path.isfile(os.path.join(self.local, self._CONTROL_FILE)):
            f    = File(os.path.join(self.local, self._CONTROL_FILE))
            data = json.loads(f.read())
            f.close()
        else:
            data = self._RELEASE
        return data

    @property
    def items(self):
        '''
        Loops through the items of the database

        @yields: Object depending on the database.
        '''
        if not self.has_local:
            SBIg.throw(self, 'A local database needs to be build first', IOError)

        for ifile in self._ITEM_FILES:
            ifile = os.path.join(self.local, ifile)
            f     = File(ifile)
            for line in f.read():
                yield self._DBOBJECT.grab(line.strip())
            f.close()

    ############
    # BOOLEANS #
    ############
    @property
    def has_local(self):
        '''
        Checks whether we already have a local database.

        @return: {Boolean}
        '''
        if not bool(len(self._MANDATORY_FILES)):
            return os.path.isdir(self.local)
        else:
            for mfile in self._MANDATORY_FILES:
                if not os.path.isfile(os.path.join(self.local, mfile)):
                    return False
            return True

    ###########
    # METHODS #
    ###########
    def create(self):
        '''
        Create the local database.
        Returns _True_ on success, _False_ otherwise

        @return: {Boolean}
        '''
        if self.has_local:
            SBIg.warn(self, 'A local copy exist. Executing update.')
            return self.update()

        self._download_sources()
        self._process()
        self._save_release()
        # self._clean_sources()

        return True

    def update(self):
        '''
        Update the local database.
        Returns _True_ on success, _False_ otherwise

        @return: {Boolean}
        '''
        self._RELEASE = self.release
        self._RELEASE['new_items']     = {}
        self._RELEASE['update_items']  = {}
        self._RELEASE['deleted_items'] = {}
        self._download_sources()
        self._process(True)
        self._save_release()
        # self._clean_sources()

        return True

    ###################
    # PRIVATE METHODS #
    ###################
    @abstractmethod
    def _process(self, update = False):
        '''
        Transform the source files into the final local db files.

        @param:    update
        @pdef:     toggles between create and update processing
        @pdefault: _False_
        @ptype:    {Boolean}
        '''
        raise NotImplemented

    def _save_release(self):
        '''
        Store the release data into a file.
        '''
        f    = File(os.path.join(self.local, self._CONTROL_FILE), 'w', True)
        f.write(json.dumps(self._RELEASE))
        f.close()

    def _download_sources(self):
        '''
        Download the source files to local directory.
        '''
        for dfile in self._SOURCES:
            download    = False
            source      = os.path.join(self._FTP, dfile)
            source_size = DBlink._file_size(source)
            # source_date = DBlink._file_date(source)
            DBlink._SOURCES_SIZES.append(source_size)
            destination = os.path.join(self.local, dfile)
            if not os.path.isfile(destination):
                if DBlink._RELEASE['date'] != DBlink._TODAY:
                    # if source_date < DBlink._RELEASE:
                    #     SBIg.alert('verbose', self, 'No new updates in the source side' +
                    #                                 ' for {0}'.format(source) +
                    #                                 ' since the last local update.')
                    # else:
                    download = True
                else:
                    download = True
            else:
                SBIg.alert('verbose', self, 'Looks like {0} has already been downloaded.'.format(source))

            if download:
                SBIg.alert('verbose', self, 'Downloading {0} to {1}'.format(source, destination))
                SBIg.alert('verbose', self, 'Source file size is {0:.3f} MB.'.format(source_size))
                urllib.urlretrieve(source, destination)

    def _clean_sources(self):
        '''
        Remove source files.
        '''
        for dfile in self._SOURCES:
            destination = os.path.join(self.local, dfile)
            if not SBIg.debug:
                os.unlink(destination)

    @staticmethod
    def _file_size(external_file):
        '''
        Determines the size of a remote file. In Mega Bytes.

        @param:    external_file
        @pdef:     remote file
        @ptype:    {String}

        @return: {Float}
        '''
        usock = urllib.urlopen(external_file)
        size  = usock.info().get('Content-Length')
        if size is None:
            return 0
        return float(size) / (1024.0 * 1024.0)

    # @staticmethod
    # def _file_date(external_file):
    #     '''
    #     Determines the date of a remote file.

    #     @param:    external_file
    #     @pdef:     remote file
    #     @ptype:    {String}

    #     @return: {Time}
    #     '''
    #     usock = urllib.urlopen(external_file)
    #     lastm = usock.info().get('last-modified')
    #     return time.strptime(lastm, '%a, %d %b %Y %H:%M:%S %Z')



import math, numpy

def get_points_distance(point1, point2):
    '''
    Get the distance between two points
    '''
    point1 = numpy.array(point1)
    point2 = numpy.array(point2) 
    diff = point1-point2
    return numpy.sqrt(numpy.dot(diff, diff))

def translate_point(point, direction):
    '''
    '''
    x = point[0] + direction[0]
    y = point[1] + direction[1]
    z = point[2] + direction[2]
    return x, y, z

###########
#   HEX   #
###########

_AXES2TUPLE = { 'sxyz': (0, 0, 0, 0), 'sxyx': (0, 0, 1, 0), 'sxzy': (0, 1, 0, 0),
                'sxzx': (0, 1, 1, 0), 'syzx': (1, 0, 0, 0), 'syzy': (1, 0, 1, 0),
                'syxz': (1, 1, 0, 0), 'syxy': (1, 1, 1, 0), 'szxy': (2, 0, 0, 0),
                'szxz': (2, 0, 1, 0), 'szyx': (2, 1, 0, 0), 'szyz': (2, 1, 1, 0),
                'rzyx': (0, 0, 0, 1), 'rxyx': (0, 0, 1, 1), 'ryzx': (0, 1, 0, 1),
                'rxzx': (0, 1, 1, 1), 'rxzy': (1, 0, 0, 1), 'ryzy': (1, 0, 1, 1),
                'rzxy': (1, 1, 0, 1), 'ryxy': (1, 1, 1, 1), 'ryxz': (2, 0, 0, 1),
                'rzxz': (2, 0, 1, 1), 'rxyz': (2, 1, 0, 1), 'rzyz': (2, 1, 1, 1) }

_TUPLE2AXES = dict((v, k) for k, v in _AXES2TUPLE.items())

# axis sequences for Euler angles
_NEXT_AXIS = [1, 2, 0, 1]

# epsilon for testing whether a number is close to zero
_EPS = numpy.finfo(float).eps * 4.0

def get_hex_rotation_matrix(angles, axes='szyz'):
    '''
    ai, aj, ak : Euler's roll, pitch and yaw angles
    axes : One of 24 axis sequences as string or encoded tuple (default HEX: szyz)
    '''
    try:
        firstaxis, parity, repetition, frame = _AXES2TUPLE[axes.lower()]
    except (AttributeError, KeyError):
        _TUPLE2AXES[axes]  # validation
        firstaxis, parity, repetition, frame = axes

    i = firstaxis
    j = _NEXT_AXIS[i+parity]
    k = _NEXT_AXIS[i-parity+1]

    ai = -angles[0]
    aj = -angles[1]
    ak = -angles[2]

    if frame:
        ai, ak = ak, ai
    if parity:
        ai, aj, ak = -ai, -aj, -ak

    si, sj, sk = math.sin(ai), math.sin(aj), math.sin(ak)
    ci, cj, ck = math.cos(ai), math.cos(aj), math.cos(ak)
    cc, cs = ci*ck, ci*sk
    sc, ss = si*ck, si*sk

    M = numpy.identity(3)
    if repetition:
        M[i, i] = cj
        M[i, j] = sj*si
        M[i, k] = sj*ci
        M[j, i] = sj*sk
        M[j, j] = -cj*ss+cc
        M[j, k] = -cj*cs-sc
        M[k, i] = -sj*ck
        M[k, j] = cj*sc+cs
        M[k, k] = cj*cc-ss
    else:
        M[i, i] = cj*ck
        M[i, j] = sj*sc-cs
        M[i, k] = sj*cc+ss
        M[j, i] = cj*sk
        M[j, j] = sj*ss+cc
        M[j, k] = sj*cs-sc
        M[k, i] = -sj
        M[k, j] = cj*si
        M[k, k] = cj*ci

    return M

def hex_transform_point(point, direction, angles):
    '''
    '''
    rotation_matrix = get_hex_rotation_matrix(angles)
    rotated_point = numpy.dot(point, rotation_matrix)
    return translate_point(rotated_point, direction)

############
# PACHDOCK #
############

def get_pachdock_rotation_matrix(angles):
    '''
    '''
    rotation_matrix = numpy.identity(3)

    cos_x = math.cos(angles[0])
    sin_x = math.sin(angles[0])
    cos_y = math.cos(angles[1])
    sin_y = math.sin(angles[1])
    cos_z = math.cos(angles[2])
    sin_z = math.sin(angles[2])

    rotation_matrix[0][0] = cos_z * cos_y
    rotation_matrix[1][0] = sin_z * cos_y
    rotation_matrix[2][0] = sin_y   
    rotation_matrix[0][1] = -sin_y * sin_x * cos_z - sin_z * cos_x
    rotation_matrix[1][1] = -sin_y * sin_x * sin_z + cos_x * cos_z
    rotation_matrix[2][1] = cos_y * sin_x
    rotation_matrix[0][2] = -sin_y * cos_x * cos_z + sin_z * sin_x
    rotation_matrix[1][2] = -sin_y * cos_x * sin_z - sin_x * cos_z
    rotation_matrix[2][2] = cos_y * cos_x

    return rotation_matrix

def pachdock_transform_point(point, direction, angles):
    '''
    transformed_point = rotation_matrix*point + direction
    '''
    rotation_matrix = get_pachdock_rotation_matrix(angles)
    rotated_point = numpy.dot(rotation_matrix, point)
    return translate_point(rotated_point, direction)

#def rotate_point_euclidean(point, angles):
#
#    '''
#    Rotate a 3D point using euclidean angles
#    '''
# 
#    rotation_matrix = numpy.zeros([3,3], float)
#
#    cos_x = math.cos(numpy.radians(angles[0]))
#    sin_x = math.sin(numpy.radians(angles[0]))
#    cos_y = math.cos(numpy.radians(angles[1]))
#    sin_y = math.sin(numpy.radians(angles[1]))
#    cos_z = math.cos(numpy.radians(angles[2]))
#    sin_z = math.sin(numpy.radians(angles[2]))
#
#    rotation_matrix[0][0] = cos_y * cos_z
#    rotation_matrix[1][0] = cos_y * sin_z
#    rotation_matrix[2][0] = -sin_y
#
#    rotation_matrix[0][1] = -cos_x * sin_z + sin_x * sin_y * cos_z
#    rotation_matrix[1][1] = cos_x * cos_z + sin_x * sin_y * sin_z
#    rotation_matrix[2][1] = sin_x * cos_y
#
#    rotation_matrix[0][2] = sin_x * sin_z + cos_x * sin_y * cos_z
#    rotation_matrix[1][2] = -sin_x * cos_z + cos_x * sin_y * sin_z
#    rotation_matrix[2][2] = cos_x * cos_y
#
#    x = rotation_matrix[0][0] * point[0] + rotation_matrix[0][1] * point[1] + rotation_matrix[0][2] * point[2]
#    y = rotation_matrix[1][0] * point[0] + rotation_matrix[1][1] * point[1] + rotation_matrix[1][2] * point[2]
#    z = rotation_matrix[2][0] * point[0] + rotation_matrix[2][1] * point[1] + rotation_matrix[2][2] * point[2]
#
#    return x, y, z
#
#def rotate_point_twists(point, angles):
#
#    '''
#    Rotate a 3D point using the Z, theta and phi twists (used by FTDock)
#    '''
#
#    #Z axis twist
#    z_twist = numpy.radians(angles[0])
#    z_twist_x = point[0] * math.cos(z_twist) - point[1] * math.sin(z_twist)
#    z_twist_y = point[0] * math.sin(z_twist) + point[1] * math.cos(z_twist)
#    z_twist_z = point[2]
#    #theta twist along plane of x-z
#    theta = numpy.radians(angles[1])
#    theta_x = z_twist_z * math.sin(theta) + z_twist_x * math.cos(theta)
#    theta_y = z_twist_y
#    theta_z = z_twist_z * math.cos(theta) - z_twist_x * math.sin(theta)
#    #phi twist around z axis
#    phi = numpy.radians(angles[2])
#    phi_x = theta_x * math.cos(phi) - theta_y * math.sin(phi)
#    phi_y = theta_x * math.sin(phi) + theta_y * math.cos(phi)
#    phi_z = theta_z
#
#    x = phi_x
#    y = phi_y
#    z = phi_z
#
#    return x, y, z



"""
PHInterface

author: jbonet
date:   03/2013

@oliva's lab
"""
from .         import Interface
from ..contact import ContactAH

class PHInterface(Interface):
    """
    {Interface} between two protein chains
    """
    def __init__(self, protein         = None,  heteroatom          = None, 
                       threshold_type  = "min", threshold_distance = 8,
                       protein_centers = None,  heteroatom_centers  = None):

        if protein_centers is None:
            protein_centers    = protein.geometric_center(structure = True,  hetero     = False,
                                                          water     = False, by_residue = True)
        if heteroatom_centers is None:
            heteroatom_centers = heteroatom.geometric_center(structure = False, hetero     = True,
                                                             water     = False, by_residue = True)

        super(PHInterface, self).__init__(chain1         = protein,         chain2             = heteroatom,
                                          threshold_type = threshold_type,  threshold_distance = threshold_distance,
                                          chain1_centres = protein_centers, chain2_centers     = heteroatom_centers)

    #
    # ATTRIBUTES
    #
    @property
    def protein(self):             return self._chain1
    @property
    def protein_id(self):          return self._chain1.globalID
    @property
    def protein_centers(self):     return self._chain1_centers

    @property
    def heteroatoms(self):         return self._chain2
    @property
    def heteroatoms_id(self):      return self._chain2.globalID
    @property
    def heteroatoms_centers(self): return self._chain1_centers

    @property
    def protein_positions(self):
        return self._list_positions(1)
    @property 
    def protein_view_interface(self):
        return self._view_interface_from(1)

    @property
    def heteroatoms_positions(self):
        return self._list_positions(2)
    @property 
    def heteroatoms_view_interface(self):
        return self._view_interface_from(2)

    #
    # OVERWRITE PARENT: PRIVATE FUNCTIONS
    #
    def _build(self):
        if len(self.protein.aminoacids) == 0 or len(self.heteroatoms.heteroatoms) == 0: return
        super(PHInterface, self)._build()

        for i in range(len(self._filtered[0])):
            new_contact = ContactAH(aminoacid          = self.protein.aminoacids[self._filtered[0][i]], 
                                    heteroatom         = self.heteroatoms.heteroatoms[self._filtered[1][i]], 
                                    threshold_type     = self.threshold_type, 
                                    threshold_distance = self.threshold_distance)
            if new_contact.is_underthreshold:
                self.contacts = new_contact

    #
    # OVERWRITE PARENT: TOSTRING
    #
    def toString(self, all_types = False):
        data = []
        if not all_types:
            data.append('{0._chain1.chain}\t{0._chain2.chain}\t{0.threshold_type}\t{0.threshold_distance}'.format(self))
        else:
            data.append('{0._chain1.chain}\t{0._chain2.chain}\tmin\tgeometric'.format(self))
        data.append(super(PHInterface, self).toString(all_types))
        return "\n".join(data)


import sys, os

class Interaction(object):
    '''
    This class represent an interaction between two structures
    '''
 
    def __init__(self, structure1, structure2):
        '''
        Constructor
        '''
        self.structure1 = structure1
        self.structure2 = structure2
        self.positive_signatures = []
        self.negative_signatures = []

    # GETTERS #

    def get_structure1(self):
        return self.structure1

    def get_structure2(self):
        return self.structure2
   
    def get_structures(self):
        return self.structure1, self.structure2

    def get_positive_signatures(self):
        return self.positive_signatures

    def get_negative_signatures(self):
        return self.negative_signatures

    # SETTERS #

    def set_positive_signatures(self, positive_signatures):
        self.positive_signatures = positive_signatures

    def set_negative_signatures(self, negative_signatures):
        self.negative_signatures = negative_signatures

    # CLASS METHODS #

    def get_interface_signatures(self, min_coverage = 0.8, c_type = "ca", max_distance = 8, max_pval = 1, signature_type = 'all'):
        '''
        Get the signatures in the interface and the total combination of signatures (used to normalize)
        NOTE: We assume the signatures are ordered from better to worst pValue
        NOTE: We assume structure 1 contains the loops of the first protein signature: (str1 loops, str2loops, pValue)
        '''
        # Check signature type #
        signature_type_accepted = set(['all','contact','no_contact'])
        if signature_type not in signature_type_accepted:
            signature_type = 'all'
        # Init local variables #
        positive_signatures = []
        negative_signatures = []
        total_combination = 0
        # Ger interacting loops of each strcuture #
        interacting_loops_structure1 = self.get_structure1().get_interacting_loops(self.get_structure2(), min_coverage, c_type, max_distance)
        interacting_loops_structure2 = self.get_structure2().get_interacting_loops(self.get_structure1(), min_coverage, c_type, max_distance)
        # Iterate over loops combinations #
        for loop1 in interacting_loops_structure1:
            for loop2 in interacting_loops_structure2:
                if signature_type == 'all' or (signature_type == 'contact' and loop1.get_interacting_coverage(loop2) > 0) or (signature_type == 'no_contact' and loop1.get_interacting_coverage(loop2) == 0):
                    total_combination += 1
                    for positive_signature in self.get_positive_signatures():
                        if loop1.get_id() in positive_signature[0] and loop2.get_id() in positive_signature[1]:
                            if positive_signature[2] <= max_pval:
                                positive_signatures.append(positive_signature)
                                break
                    for negative_signature in self.get_negative_signatures():
                        if loop1.get_id() in negative_signature[0] and loop2.get_id() in negative_signature[1]:
                            if negative_signature[2] <= max_pval:
                                negative_signatures.append(negative_signature)
                                break
        return positive_signatures, negative_signatures, total_combination

    def get_partial_signatures(self, min_coverage = 0.8, c_type = "ca", max_distance = 8, max_pval = 1):
        '''
        Get the signatures in the partial location and the total combination of signatures (used to normalize)
        NOTE: We assume the signatures are ordered from better to worst pValue
        NOTE: We assume structure 1 contains the loops of the first protein signature: (str1 loops, str2loops, pValue)
        '''
        positive_signatures = []
        negative_signatures = []
        total_combination = 0
        interacting_loops_structure1 = self.get_structure1().get_interacting_loops(self.get_structure2(), min_coverage, c_type, max_distance)
        noninteracting_loops_structure1 = self.get_structure1().get_noninteracting_loops(self.get_structure2(), min_coverage, c_type, max_distance)
        interacting_loops_structure2 = self.get_structure2().get_interacting_loops(self.get_structure1(), min_coverage, c_type, max_distance)
        noninteracting_loops_structure2 = self.get_structure2().get_noninteracting_loops(self.get_structure1(), min_coverage, c_type, max_distance)
        for loop1 in noninteracting_loops_structure1:
            for loop2 in interacting_loops_structure2:
                total_combination += 1
                for positive_signature in self.get_positive_signatures():
                    if loop1.get_id() in positive_signature[0] and loop2.get_id() in positive_signature[1]:
                        if positive_signature[2] <= max_pval:
                            positive_signatures.append(positive_signature)
                            break
                for negative_signature in self.get_negative_signatures():
                     if loop1.get_id() in negative_signature[0] and loop2.get_id() in negative_signature[1]:
                        if negative_signature[2] <= max_pval:
                            negative_signatures.append(negative_signature)
                            break
        for loop1 in interacting_loops_structure1:
            for loop2 in noninteracting_loops_structure2:
                total_combination += 1
                for positive_signature in self.get_positive_signatures():
                    if loop1.get_id() in positive_signature[0] and loop2.get_id() in positive_signature[1]:
                        if positive_signature[2] <= max_pval:
                            positive_signatures.append(positive_signature)
                            break
                for negative_signature in self.get_negative_signatures():
                     if loop1.get_id() in negative_signature[0] and loop2.get_id() in negative_signature[1]:
                        if negative_signature[2] <= max_pval:
                            negative_signatures.append(negative_signature)
                            break
        return positive_signatures, negative_signatures, total_combination

    def get_external_signatures(self, min_coverage = 0.8, c_type = "ca", max_distance = 8, max_pval = 1):
        '''
        Get the signatures in the external location and the total combination of signatures (used to normalize)
        NOTE: We assume the signatures are ordered from better to worst pValue
        NOTE: We assume structure 1 contains the loops of the first protein signature: (str1 loops, str2loops, pValue)
        '''
        positive_signatures = []
        negative_signatures = []
        total_combination = 0
        noninteracting_loops_structure1 = self.get_structure1().get_noninteracting_loops(self.get_structure2(), min_coverage, c_type, max_distance)
        noninteracting_loops_structure2 = self.get_structure2().get_noninteracting_loops(self.get_structure1(), min_coverage, c_type, max_distance)
        for loop1 in noninteracting_loops_structure1:
            for loop2 in noninteracting_loops_structure2:
                total_combination += 1
                for positive_signature in self.get_positive_signatures():
                    if loop1.get_id() in positive_signature[0] and loop2.get_id() in positive_signature[1]:
                        if positive_signature[2] <= max_pval:
                            positive_signatures.append(positive_signature)
                            break
                for negative_signature in self.get_negative_signatures():
                     if loop1.get_id() in negative_signature[0] and loop2.get_id() in negative_signature[1]:
                        if negative_signature[2] <= max_pval:
                            negative_signatures.append(negative_signature)
                            break
        return positive_signatures, negative_signatures, total_combination 

    def get_interacting_residues(self, c_type = "ca", max_distance = 8):
        '''
        Get a list of the interacting residues of both structures and distances
        '''
        interacting_residues_structure1 = self.get_structure1().get_interacting_residues(self.get_structure2(), c_type, max_distance)
        interacting_residues_structure2 = self.get_structure2().get_interacting_residues(self.get_structure1(), c_type, max_distance)    
        return interacting_residues_structure1, interacting_residues_structure2

    def __str__(self):
        '''
        Return the interacting residues tab separated
        '''
        pass



"""
Interface

author: jbonet
date:   02/2014

@oliva's lab
"""
import copy, hashlib
import scipy.spatial as sp
import numpy         as np

class Interface(object):
    """docstring for Interface"""
    def __init__(self, chain1, chain2, threshold_type, threshold_distance, chain1_centres = None, chain2_centers = None):
        
        self._chain1             = chain1
        c1                       = chain1.globalID
        self._chain1_centers     = chain1_centres
        self._chain2             = chain2
        c2                       = chain2.globalID
        self._chain2_centers     = chain2_centers
        self._md5                = tuple(sorted([hashlib.sha224(c1 + c2).hexdigest(),
                                                 hashlib.sha224(c2 + c1).hexdigest()])).__hash__()
        self._threshold_type     = threshold_type
        self._threshold_distance = threshold_distance

        self._contacts           = set()

        self._filterdistance     = threshold_distance if threshold_type is "geometric" else threshold_distance + 15
        self._filtered           = None

        self._build()

    #
    # ATTRIBUTES
    #
    @property 
    def identifier(self):         return self._md5

    @property
    def threshold_type(self):     return self._threshold_type

    @property
    def threshold_distance(self): return self._threshold_distance

    @property
    def contacts(self):           return self._contacts
    @contacts.setter
    def contacts(self, contact):  self._contacts.add(contact)

    #
    # METHODS
    #
    @staticmethod
    def test_identifier(chain1, chain2):
        c1 = chain1.globalID
        c2 = chain2.globalID
        return tuple(sorted([hashlib.sha224(c1 + c2).hexdigest(),
                             hashlib.sha224(c2 + c1).hexdigest()])).__hash__()

    def reverse(self):
        (self._chain1, self._chain2) = (self._chain2, self._chain1)
        map(lambda x: x.reverse(), self._contacts)

    #
    # PRIVATE METHODS
    #
    def _build(self):
        if self._chain1_centers is not None and self._chain2_centers is not None:
            distance_matrix = sp.distance.cdist(self._chain1_centers, self._chain2_centers)
            self._filtered  = np.where(distance_matrix <= self._filterdistance)

    def _list_positions(self, chain):
        positions = set()
        for c in self.contacts:
            if chain   == 1: r = c._residue1
            elif chain == 2: r = c._residue2
            positions.add(r)
        return positions

    def _view_interface_from(self, chain):
        positions = {}
        for c in self.contacts:
            if chain   == 1: r = c._residue1
            elif chain == 2: r = c._residue2
            positions.setdefault(r, []).append(c)
        return positions

    #
    # OVERRIDE DEFAULT METHODS
    #
    def __len__(self): return len(self._contacts)
    def __add__(self, other):
        if self.threshold_type != other.threshold_type:
            raise AttributeError("Different threshold type interfaces can not be added\n")
        if not isinstance(other, self.__class__):
            raise AttributeError("Different types of interfaces can not be joined\n")
        if self._chain1.globalID == other._chain1.globalID  and self._chain2.globalID  == other._chain2.globalID:
            self._contacts.update(other._contacts)
            self._threshold_distance = self.threshold_distance if self.threshold_distance > other.threshold_distance else other.threshold_distance
        elif self._chain1.globalID == other._chain2.globalID  and self._chain2.globalID  == other._chain1.globalID:
            t = copy.deepcopy(other)
            t.reverse()
            self.__add__(t)
        else:
            raise AttributeError("Interfaces from different chains can not be added\n")
    
    #
    # TOSTRING
    #
    def toString(self, all_types = False):
        data = []
        # data.append('{0._chain1.chain}\t{0._chain2.chain}\t{0.threshold_type}\t{0.threshold_distance}'.format(self))
        for contact in self.contacts:
            if not all_types:
                data.append(contact.toString(self.threshold_type))
            else:
                data.append(contact.toString("all"))
        return "\n".join(data)




import sys
import os
import numpy as np
import matplotlib as mpl
#mpl.use('Agg')
import matplotlib.pyplot as plt
from matplotlib.patches import Polygon
import string
import argparse
from scipy import stats

def parse_user_arguments(*args, **kwds):
 parser = argparse.ArgumentParser(
  description = "Create a BOXPLOT with a list of data files",
  epilog      = "@oliva's lab 2014")
 parser.add_argument('-l','--list_input_file',dest='input',action = 'store',default='input_list',
  help = 'Input file (default is input_list)')
 parser.add_argument('-f','--format_image',dest='format',action = 'store',default='png',
  help = 'Format of the image in the output file: png(default), pdf, ps, eps and svg ')
 parser.add_argument('-o','--output_image',dest='out',action = 'store',default='output',
  help = 'Output file name for the imag boxplot')
 parser.add_argument('-top','--Top_Y_value',dest='top',action = 'store',default=0,type=float,
  help = 'Top value of Y-axis')
 parser.add_argument('-bottom','--Botom_Y_value',dest='bottom',action = 'store',default=0,type=float,
  help = 'Bottom value of Y-axis')
 parser.add_argument('-s','--show_canvas',dest='canvas',action = 'store_true',
  help = 'Show canvas figure after execution (not shown by default)')
 parser.add_argument('-g','--show_grid',dest='grid',action = 'store_true',
  help = 'Add horizontal gridlines (default is without)')
 parser.add_argument('-legend','--show_legend',dest='legend',action = 'store_true',
  help = 'Add legends (default is without)')
 parser.add_argument('-t','--title_of_plot',dest='title',action = 'store',default='Boxplot',
  help = 'Title of the plot within "" ')
 parser.add_argument('-nc','--set_colored_groups',dest='color',action = 'store',type=int, default=1,
  help = 'Number of sets grouped with the same color. Up to 14 different colors (default 1 => every boxplot has different color)')
 parser.add_argument('-nh','--set_hashed_groups',dest='hashed',action = 'store',type=int, default=1,
  help = 'Number of sets grouped with the same filling hash. Up to 10 different filling hashes (default 1 => every boxplot has different hash)')
 parser.add_argument('-v','--verbose',dest='show',action = 'store_true',
  help = 'Verbose execution')

 options=parser.parse_args()
 return options

def fileExist (file):
 return os.path.exists(file) and os.path.isfile(file)

def printverbose(f,flag,message):
 """Define verbose to print in file 'f', if 'flag'=True, message given as 'message'"""
 if flag: f.write("%s"%(message))

def main():
 options = parse_user_arguments()
 try:
  boxplotlist(options)
 except IOError as e:
  print("I/O error (%s): %s" %(e.errno, e.strerror))

def boxplotlist(options):

 if fileExist(options.input):
  dataname=[]
  dataplot=[]
  fp=open(options.input,"r")
  for line in fp:
   (name,datafile)=line.strip().split()
   if fileExist(datafile):
     dataname.append(name)
     datavalue=[]
     fi=open(datafile,"r")
     printverbose(sys.stdout,options.show,"Open %s\n"%(datafile))
     for datainfo in fi:
       (info,value)=datainfo.strip().split()
       datavalue.append(float(value))
     dataplot.append(np.array(datavalue))
     fi.close()
  fp.close()
 else:
  raise IOError("Input File not found")

 fig, ax1 = plt.subplots(figsize=(10,6))
 title='Boxplot of '+ options.input
 fig.canvas.set_window_title(title)
 plt.subplots_adjust(left=0.075, right=0.95, top=0.9, bottom=0.25)

 printverbose(sys.stdout,options.show,"Data %s %s\n"%(repr(dataname),repr(dataplot)))
 bp = plt.boxplot(dataplot, notch=0, sym='+', vert=1, whis=1.5)
 plt.setp(bp['boxes'], color='black')
 plt.setp(bp['whiskers'], color='black')
 plt.setp(bp['fliers'], color='red', marker='+')

# Add a horizontal grid to the plot, but make it very light in color
# so we can use it for reading data values but not be distracting
 if options.grid:
   ax1.yaxis.grid(True, linestyle='-', which='major', color='lightgrey',
              alpha=0.5)

# Hide these grid behind plot objects
 ax1.set_axisbelow(True)

# Add Title of the plot
 ax1.set_title(options.title)

 ax1.set_xlabel('Samples')
 ax1.set_ylabel('Value')

# Fill the boxes with desired colors and hashes
 boxColors = ['white','darkkhaki','royalblue','salmon','lime','red','violet','cyan','blue',
             'yellow','green','navy','grey','orange','firebrick']
 boxHatch  = [' ','/','\\','|','-','+','x','o','.','*']

 if options.color==0: options.color=1
 if options.hashed==0: options.hashed=1
 numBoxes = len(dataplot)
 medians = range(numBoxes)
 k=0
 kk=0
# Statistics: compare all vs all data
 ks={}
 mw={} 
 for i in range(numBoxes):
  for j in range(numBoxes):
   try:
    ks.setdefault((i,j),stats.ks_2samp(dataplot[i],dataplot[j]))
   except:
    ks.setdefault((i,j),(0.0,1.0))
    e = sys.exc_info()[0]
    sys.stdout.write("Fail Kolmogorov Smirnov Test %s\n"%repr(e))
   try:
    mw.setdefault((i,j),stats.mannwhitneyu(dataplot[i],dataplot[j]))
   except:
    mw.setdefault((i,j),(0.0,0.5))
    e = sys.exc_info()[0]
    sys.stdout.write("Fail Mann-Whitney Test %s\n"%e)

#Boxplot
 for i in range(numBoxes):
   box = bp['boxes'][i]
   boxX = []
   boxY = []
   for j in range(5):
      boxX.append(box.get_xdata()[j])
      boxY.append(box.get_ydata()[j])
   boxCoords = zip(boxX,boxY)
  # Group by color and hashed lines
   n= k * options.color + i % options.color
   if n!=i:
    k=k+1
    if k>14: k=0
    kk=0
   j= i - k * options.color
   m= kk * options.hashed + j %  options.hashed
   if m!=j:
    kk=kk+1
    if kk>9: kk=0
   boxPolygon = Polygon(boxCoords, facecolor=boxColors[k], hatch= boxHatch[kk])
   ax1.add_patch(boxPolygon)
  # Now draw the median lines back over what we just filled in
   med = bp['medians'][i]
   medianX = []
   medianY = []
   for j in range(2):
       medianX.append(med.get_xdata()[j])
       medianY.append(med.get_ydata()[j])
       plt.plot(medianX, medianY, 'black',linewidth=2.5)
       medians[i] = medianY[0]
  # Finally, overplot the sample averages, with horizontal alignment
  # in the center of each box
   plt.plot([np.average(med.get_xdata())], [np.average(dataplot[i])],
           color='white', marker='*', markeredgecolor='black')

# Set the axes ranges and axes labelsa
 ax1.set_xlim(0.5, numBoxes+0.5)
 if options.top ==0:
  top = max([ max([x for x in dataplot[i]]) for i in range(numBoxes) ])
 else:
  top = options.top
 if options.bottom==0:
  bottom = min([ min([x for x in dataplot[i]]) for i in range(numBoxes) ])
 else:
  bottom=options.bottom
 ax1.set_ylim(bottom, top)
 xtickNames = plt.setp(ax1, xticklabels=dataname)
 plt.setp(xtickNames, rotation=45, fontsize=12)

# Finally, add a basic legend
 if options.legend:
  posy=0.12
  posx=0.7
  k=0
  for i in range(numBoxes):
   # Group by color and hashed lines
    n= k * options.color + i % options.color
    if n!=i:
     k=k+1
    plt.figtext(posx, posy,  dataname[i] ,
            backgroundcolor=boxColors[k], color='black', weight='roman',
            size='x-small')
    posy=posy-0.03
    if posy<0.01:
     posy=0.12
     posx=posx+0.12

  plt.figtext(0.14, 0.1, '*', color='white', backgroundcolor='silver',
           weight='roman', size='medium')
  plt.figtext(0.15, 0.095, ' Average Value', color='black', weight='roman',
           size='x-small')

 output=options.out+"."+options.format
 printverbose(sys.stdout,options.show,"output is %s\n"%output)
 plt.savefig(output)

 sys.stdout.write("Kolmogorov-Smirnov Statistic\n") 
 sys.stdout.write("+--------------------------+\n") 
 for i in range(numBoxes):
  for j in range(numBoxes):
    if j>=i :
     sys.stdout.write("%s versus %s : Statistic %f p-value %f Average(%s)= %f Average(%s)= %f\n"%(dataname[i],dataname[j],ks[(i,j)][0],ks[(i,j)][1],dataname[i],np.average(dataplot[i]),dataname[j],np.average(dataplot[j])))


 sys.stdout.write("\nMann-Whitney U Statistic\n") 
 sys.stdout.write("+----------------------+\n") 
 for i in range(numBoxes):
  for j in range(numBoxes):
    if j>=i :
     sys.stdout.write("%s versus %s : Statistic %f p-value %f Average(%s)= %f Average(%s)= %f\n"%(dataname[i],dataname[j],mw[(i,j)][0],2*mw[(i,j)][1],dataname[i],np.average(dataplot[i]),dataname[j],np.average(dataplot[j])))


     
 if options.canvas:
  plt.show()


if __name__=="__main__":
 main()






from SBI.beans.IndexedNum import IndexedNum
from SBI.beans.JSONer     import JSONer


class MiniResidue(JSONer):
    def __init__(self, restype, position):
        self._type  = restype
        self._chain = None
        self._pos   = None
        self._parse_position(position)

    #
    # ATTRIBUTES
    #
    @property
    def restype(self):
        return self._type

    @property
    def chain(self):
        return self._chain

    @property
    def position(self):
        return int(self._pos)

    @property
    def idxp(self):
        return self._pos.index

    def _parse_position(self, position):
        self._chain = position[0]
        self._pos   = IndexedNum(position[1:].strip())

    #
    # OUT FUNCTION
    #
    def as_dict(self):
        nobj = {}
        nobj['type']  = self.restype
        nobj['chain'] = self.chain
        nobj['pos']   = self.position
        nobj['idxp']  = self.idxp
        return nobj



import os, sys, subprocess

def get_dssp_exec():
    '''
    Gets the DSSP executable
    '''
    try:
        dssp_path = os.environ['DSSP_PATH']
    except:
        sys.stderr.write('The $DSSP_PATH environment variable is not defined, trying local library dssp\n')
        return None
    return os.path.join(dssp_path, 'dsspcmbi')

def execute_dssp(structure):
    '''
    Executes dssp and sets secondary structure (ss) and accesible surface area (acc) to each residue of the given structure
    '''
    dssp_exec = get_dssp_exec()
    if not dssp_exec:
        dssp_exec = os.path.abspath(os.path.join(os.path.dirname(__file__), 'dssp'))
    if not os.path.exists(dssp_exec):
        sys.stderr.write('Cannot call dssp, skipping...\n')
        return None

    p = subprocess.Popen([dssp_exec, '--'], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    dssp_out, dssp_err = p.communicate(str(structure))

    read = False
    for line in dssp_out.split('\n'):
        if line.startswith('  #  RESIDUE AA STRUCTURE BP1 BP2  ACC'):
            read = True
            continue
        if read and line != '' and line[13] != '!':
            residue_num = int(line[6:10].strip())
            ss = line[16:17]
            acc = int(line[35:38].strip())
            if ss != "H" and ss != "E":
                ss = "C"
            residue = structure.get_residue_by_num(residue_num)
            residue.set_ss(ss)
            residue.set_acc(acc)


'''
@file: HmmHit.py

@author: Jaume Bonet
@mail:   jaume.bonet@gmail.com
@date:   2014

@ [oliva's lab](http://sbi.imim.es)

@class: HmmHit
'''


class HmmHit(object):
    '''
    Stores the data of each hit output of HMMER
    '''
    def __init__(self, name, accession, length, query_lim, hit_lim,
                 values, repeats):
        '''
        @param:    name
        @pdef:     hit name
        @ptype:    {String}

        @param:    accession
        @pdef:     hit identifier
        @ptype:    {String}

        @param:    length
        @pdef:     length of the query protein
        @ptype:    {Integer}

        @param:    query_lim
        @pdef:     limits of the assignation in the query sequence
        @ptype:    {List} of {Integer}

        @param:    hit_lim
        @pdef:     limits of the assignation in the hit sequence
        @ptype:    {List} of {Integer}

        @param:    values
        @pdef:     evalue and ievalue
        @ptype:    {List} of {Float}

        @param:    repeats
        @pdef:     number of apparitions of this hit in the sequence
        @ptype:    {Integer}
        '''
        self._accession = accession if accession != '-' else name.split(':')[1]
        self._name      = name if accession != '-'  else name.split(':')[0]
        self._length    = length
        self._qini      = query_lim[0]
        self._qend      = query_lim[1]
        self._alength   = int(self._qend) - int(self._qini) + 1
        self._hini      = hit_lim[0]
        self._hend      = hit_lim[1]
        self._evalue    = values[0]
        self._repeats   = repeats
        self._ievalue   = values[1]
        self._coverage  = self._set_coverage()

        self._used      = False

    ##############
    # ATTRIBUTES #
    ##############
    @property
    def accession(self):
        '''
        @return: {String}
        '''
        return self._accession

    @property
    def name(self):
        '''
        @return: {String}
        '''
        return self._name

    @property
    def length(self):
        '''
        @return: {Integer}
        '''
        return int(self._length)

    @property
    def applied_length(self):
        '''
        @return: {Integer}
        '''
        return int(self._alength)

    @property
    def query_ini(self):
        '''
        @return: {Integer}
        '''
        return int(self._qini)

    @property
    def query_end(self):
        '''
        @return: {Integer}
        '''
        return int(self._qend)

    @property
    def hit_ini(self):
        '''
        @return: {Integer}
        '''
        return int(self._hini)

    @property
    def hit_end(self):
        '''
        @return: {Integer}
        '''
        return int(self._hend)

    @property
    def evalue(self):
        return float(self._evalue)

    @property
    def repeats(self):
        '''
        @return: {Integer}
        '''
        return int(self._repeats)

    @property
    def ievalue(self):
        '''
        @return: {Float}
        '''
        return float(self._ievalue)

    @property
    def coverage(self):
        '''
        @return: {Float}
        '''
        return float(self._coverage)

    @property
    def used(self):
        '''
        @return: {Boolean}
        '''
        return self._used

    @used.setter
    def used(self, value):
        '''
        @param:    value
        @pdef:     new used status
        @ptype:    {Boolean}
        '''
        self._used = value

    ###########
    # METHODS #
    ###########
    def overlap(self, domain):
        '''
        calculates the overlap of two domains over the same sequence

        @param:    domain
        @pdef:     domain to which compare
        @ptype:    {HmmHit}

        @return: {Float}
        '''
        if domain.query_ini >= self.query_ini and domain.query_end <= self.query_end:
            return 1
        if domain.query_end <= self.query_ini or domain.query_ini > self.query_end:
            return 0

        start = max(domain.query_ini, self.query_ini)
        end   = min(domain.query_end, self.query_end)

        return float(end-start)/float(domain.applied_length)

    ###################
    # PRIVATE METHODS #
    ###################
    def _set_coverage(self):
        '''
        calculates how much of the domain is assigned to the protein

        @return: {Float}
        '''
        return float(self.hit_end - self.hit_ini + 1) / float(self.length)



"""TaxIDlink

author: jbonet
date:   10/2013

@oliva's lab
"""

"""
Import Standard Libraries
"""
import sys, os, re
import subprocess
import warnings
import urllib

"""
Dependences in SBI library
"""
from SBI               import SBIglobals
from SBI.databases import taxIDftp
from SBI.beans.Path     import Path
from SBI.beans.File     import File

class TaxIDlink(object):
    """The TaxIDlink class controls the download and parsing of TaxID database

    """
    def __init__(self, local = None):
        self._local   = os.path.abspath(local)
        self.__name__ = 'databases.TaxIDlink'    # This must be included in every class for the SBIglobals.alert()

        self._nodes  = 'nodes.dmp'
        self._names  = 'names.dmp'
        self._delet  = 'delnodes.dmp'
        self._merged = 'merged.dmp'
        self._taxid  = 'taxid.gz'

        if local is not None:
            self.local = local

    """ATTRIBUTES"""
    @property
    def local(self):         return self._local
    @local.setter
    def local(self, value):
        self._local  = os.path.abspath(value)
        self._nodes  = os.path.join(self._local, self._nodes)
        self._names  = os.path.join(self._local, self._names)
        self._delet  = os.path.join(self._local, self._delet)
        self._merged = os.path.join(self._local, self._merged)
        self._taxid  = os.path.join(self._local, self._taxid)

    @property
    def localTaxIDs(self):
        taxFile = File(self._taxid, 'r')
        for tax_line in taxFile.descriptor:
            yield tax_line
        taxFile.close()

    @property
    def source(self):
        return taxIDftp['show']

    """BOOLEANS"""
    @property
    def has_local(self):    return self._local is not None

    """METHODS"""
    def download(self):
        if not self.has_local:
            raise NameError('A local TaxID database directory must be defined.')

        Path.mkdir(self.local)
        destination = os.path.join(self.local, 'taxdmp.zip')
        urllib.urlretrieve(taxIDftp['global'], destination)
        command = ['unzip', '-o', destination, '-d', self.local]
        p = subprocess.Popen(command, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
        out, err = p.communicate()

        self._process()

        return True

    def get_TaxID(self, TaxID):
        if self.has_local:
            for tax_line in self.localTaxIDs:
                if tax_line.split('\t')[0] == TaxID:
                    return tax_line

        else:
            raise NameError('A local TaxID database directory must be defined.')

    def get_TaxIDs(self, TAXset):
        if isintance(TAXset, str):
            warnings.warn('For single TaxID search the get_TaxID function is recomended.')
            yield self.get_TaxID(TAXset)

        if self.has_local:
            for tax_line in self.localTaxIDs:
                if tax_line.split('\t')[0] in TAXset:
                    yield tax_linej
        else:
            raise NameError('A local TaxID database directory must be defined.')

    """PRIVATE METHODS"""
    def _process(self):
        inh = {}
        nodefile = File(file_name = self._nodes, action = 'r')
        for line in nodefile.descriptor:
            line = re.sub('\'', '\\\'', line)
            line_data = line.split('|')
            inh[line_data[0].strip()] = TaxID(line_data[0].strip())
            inh[line_data[0].strip()].parent = line_data[1].strip()
            inh[line_data[0].strip()].rank   = line_data[2].strip()
        nodefile.close()

        namefile = File(file_name = self._names, action = 'r')
        for line in namefile.descriptor:
            line = re.sub('\'', '\\\'', line)
            line_data = line.split('|')
            if line_data[3].strip() == 'scientific name':
                inh[line_data[0].strip()].name = line_data[1].strip()
        namefile.close()

        delefile = File(file_name = self._delet, action = 'r')
        for line in delefile.descriptor:
            data = line.split('|')
            inh[data[0].strip()]     = TaxID(data[0].strip())
            inh[data[0].strip()].old = True
        delefile.close()

        mrgefile = File(file_name = self._merged, action = 'r')
        for line in mrgefile.descriptor:
            data = line.split('|')
            inh[data[0].strip()]     = TaxID(data[0].strip())
            inh[data[0].strip()].old = True
            inh[data[0].strip()].new = data[1].strip()
        mrgefile.close()

        taxFile = File(self._taxid, 'w', True)
        for taxid in inh:
            taxFile.write(str(inh[taxid]) + "\n")
        taxFile.close()

class TaxID(object):
    def __init__(self, taxid = None, inline = None):
        if inline is not None:
            inline      = inline.strip()
        self.taxid  = taxid if inline is None else inline.split('\t')[0]
        self.name   = None  if inline is None else inline.split('\t')[1] if inline.split('\t')[1] != 'None' else None
        self.rank   = None  if inline is None else inline.split('\t')[2] if inline.split('\t')[2] != 'None' else None
        self.parent = None  if inline is None else inline.split('\t')[3] if inline.split('\t')[3] != 'None' else None
        self.old    = False if inline is None else eval(inline.split('\t')[4])
        self.new    = None  if inline is None else inline.split('\t')[5] if inline.split('\t')[5] != 'None' else None

    """BOOLEANS"""
    @property
    def has_old(self): return self.old
    @property
    def has_new(self): return False if self.new is None else True

    """OVERWRITE PARENT METHODS"""
    def __str__(self):
        return "{0.taxid}\t{0.name}\t{0.rank}\t{0.parent}\t{0.old}\t{0.new}".format(self)



import os, sys, random, math
import numpy
from BioLib.Tools.BioExceptions import ResidueTriadError

class SplitPotentialsFold(object):
    '''
    Split Potentials for protein Folding. 
    Applications: Evaluate models, rank them and find incorrect folded regions.
    '''
    def __init__(self, c_type = 'CB', cutoff = 12):
        '''
        Contructor
        '''
        c_type_accepted = ('CB','MIN')
        if c_type.upper() not in c_type_accepted:
            sys.stderr.write('WARNING: Incorrect contact type %s, using CB!' % c_type)
            c_type = 'CB'

        if c_type == 'CB':
            self.sp_file = os.path.abspath(os.path.join(os.path.dirname(__file__), 'fold3_spf4_cbR12scr1gap4.out'))
        else:
            self.sp_file = os.path.abspath(os.path.join(os.path.dirname(__file__), 'fold3_spf4_minR5scr1gap4.out'))
        if not os.path.isfile(self.sp_file):
            sys.stderr.write('ERROR: Can not find the split potential file %s\n\n' % self.sp_file)
            sys.exit()

        self.c_type = c_type.upper()  # Contact type
        self.cutoff = cutoff          # Standard cutoffs are 12 for CB and 5 for MIN

        self.ppR     = {}             # Pair Potentials of Residues
        self.ppE     = {}             # Pair Potentials of Environment
        self.LppRE   = {}             # Local Pair Potentials of Residues and Environment
        self.ppRE    = {}             # Pair Potentials of Residues and Environment
        self.ppD     = {}             # Standard Pair Potentials of Distances

        self.read_sp_file()           # Read the file and load the matrices
        
    ##################
    # PUBLIC METHODS #
    ##################

    def calculate_global_energies(self, structure, Zscores=False, randoms=100):
        '''
        Compute the split potentials energies for a protein fold
        if Zscores: Computes also the Zscores
        '''
        # Compute global energies
        real_interactions = structure.get_self_interactions(c_type=self.c_type, max_distance=self.cutoff+1, gap=3)
        global_energies = self.calculate_energies(real_interactions)
        if not Zscores:
            return global_energies
        # Generate random fold energies
        rand_energies_lists = {}
        for random in xrange(randoms):
            rand_interactions = self.randomize_residue_interactions(structure.get_residues(), real_interactions)
            rand_energies = self.calculate_energies(rand_interactions)
            for potential_type in rand_energies:
                rand_energies_lists.setdefault(potential_type, []).append(rand_energies[potential_type])
        # Compute Zscores
        global_Zenergies = self.compute_Zscores(global_energies, rand_energies_lists)
        return global_energies, global_Zenergies

    def calculate_residue_energies(self, structure, Zscores=False, randoms=100):
        '''
        Compute the split potential energies for each residue in a protein fold
        if Zscores: Computes also the Zscores
        '''
        # Compute energies for each residue
        real_interactions = structure.get_self_interactions(c_type=self.c_type, max_distance=self.cutoff+1, gap=3)
        residues_energies = []
        residues_interactions = []
        for residue in structure:
            residue_interactions = []
            for real_interaction in real_interactions:
                if residue == real_interaction[0]:
                    residue_interactions.append(real_interaction)
                elif residue == real_interaction[1]:
                    residue_interactions.append((real_interaction[1], real_interaction[0], real_interaction[2]))
            residue_energies = self.calculate_energies(residue_interactions, local_type='R')
            residues_energies.append((residue, residue_energies))
            residues_interactions.append((residue, residue_interactions, residue_energies))
        if not Zscores:
            return residues_energies
        # Generate Zscore energies for each residues
        residues_energies = []
        for residue in residues_interactions:
            rand_energies_lists = {}
            for random in xrange(randoms):
                rand_interactions = self.randomize_residue_interactions(structure.get_residues(), residue[1])
                rand_energies = self.calculate_energies(rand_interactions, local_type='R')
                for potential_type in rand_energies:
                    rand_energies_lists.setdefault(potential_type, []).append(rand_energies[potential_type])
            residues_energies.append((residue[0], residue[2], self.compute_Zscores(residue[2], rand_energies_lists)))
        return residues_energies

    ########################################################
    # METHODS TO COMPUTE SPLIT POTENTIALS FOR INTERACTIONS #
    ########################################################

    def calculate_energies(self, res_interactions, local_type='G'):
        '''
        Compute the split potential energies for a set of residue-residue interactions
        '''
        local_type_accepted = ('G','R') # Global or Residue calculations
        if local_type.upper() not in local_type_accepted:
            sys.stderr.write('WARNING: Incorrect local type %s, using G!' % local_type)
            local_type = 'G'

        # Init Energy values
        energies = {'D-PAIR':   0, 'C-PAIR':  0, 'U-PAIR':  0, 'L-PAIR':  0, 'M-PAIR':  0,
                    'D-3DC':    0, 'C-3DC':   0, 'U-3DC':   0, 'L-3DC':   0, 'M-3DC':   0,
                    'D-LOCAL':  0,
                    'D-S3DC':   0, 'C-S3DC':  0, 'U-S3DC':  0, 'L-S3DC':  0, 'M-S3DC':  0,
                    'D-3D':     0, 'C-3D':    0,
                    'D-COMB':   0, 'C-COMB':  0, 'U-COMB':  0, 'L-COMB':  0, 'M-COMB':  0}

        # Compute energy for each residue interaction
        for res_interaction in res_interactions:
            residue1 = res_interaction[0]
            residue2 = res_interaction[1]
            distance = res_interaction[2]
            try:
                triad1 = residue1.get_triad()
                triad2 = residue2.get_triad()
            except ResidueTriadError as e:
                sys.stdout.write(str(e))
                sys.stdout.flush()
                continue

            # Energy of Pair Potentials of Residues #
            distance_ppR           = self.get_ppR(triad1[0:1], triad2[0:1], distance)
            energies['D-PAIR']    += distance_ppR
            cutoff_ppR             = self.get_ppR(triad1[0:1], triad2[0:1], self.cutoff)
            energies['C-PAIR']    += cutoff_ppR
            maximum_ppR            = self.get_ppR_stat(triad1[0:1], triad2[0:1], stat='upper')
            energies['U-PAIR']    += maximum_ppR
            minimum_ppR            = self.get_ppR_stat(triad1[0:1], triad2[0:1], stat='lower')
            energies['L-PAIR']    += minimum_ppR
            mean_ppR               = self.get_ppR_stat(triad1[0:1], triad2[0:1], stat='mean')
            energies['M-PAIR']    += mean_ppR
            # Energy of Pair Potentials of Environment
            distance_ppE           = self.get_ppE(triad1[2:], triad2[2:], distance)
            energies['D-3DC']     += distance_ppE
            cutoff_ppE             = self.get_ppE(triad1[2:], triad2[2:], self.cutoff)
            energies['C-3DC']     += cutoff_ppE
            maximum_ppE            = self.get_ppE_stat(triad1[2:], triad2[2:], stat='upper')
            energies['U-3DC']     += maximum_ppE
            minimum_ppE            = self.get_ppE_stat(triad1[2:], triad2[2:], stat='lower')
            energies['L-3DC']     += minimum_ppE
            mean_ppE               = self.get_ppE_stat(triad1[2:], triad2[2:], stat='mean')
            energies['M-3DC']     += mean_ppE
            # Energy of Local Pair Potentials of Residues and Environment
            if local_type == 'R':
                distance_LppRE     = self.get_LppRE(triad1)
            else:
                distance_LppRE     = self.get_LppRE(triad1) + self.get_LppRE(triad2)
            energies['D-LOCAL']   += distance_LppRE
            # Energy of Pair Potentials of Residues and Environment
            distance_ppRE          = self.get_ppRE(triad1, triad2, distance)
            energies['D-S3DC']    += distance_ppRE
            cutoff_ppRE            = self.get_ppRE(triad1, triad2, self.cutoff)
            energies['C-S3DC']    += cutoff_ppRE
            maximum_ppRE           = self.get_ppRE_stat(triad1, triad2, stat='upper')
            energies['U-S3DC']    += maximum_ppRE
            minimum_ppRE           = self.get_ppRE_stat(triad1, triad2, stat='lower')
            energies['L-S3DC']    += minimum_ppRE
            mean_ppRE              = self.get_ppRE_stat(triad1, triad2, stat='mean')
            energies['M-S3DC']    += mean_ppRE
            # Energy of Standard Pair Potentials of Distances
            distance_ppD           = self.get_ppD(distance)
            energies['D-3D']      += distance_ppD
            cutoff_ppD             = self.get_ppD(self.cutoff)
            energies['C-3D']      += cutoff_ppD
            # Energy of Combined energy
            distance_comb          = distance_ppRE + distance_ppD + distance_LppRE + distance_ppE
            energies['D-COMB']  += distance_comb
            cutoff_comb            = cutoff_ppRE + cutoff_ppD + distance_LppRE + cutoff_ppE
            energies['C-COMB']  += cutoff_comb
            maximum_comb           = maximum_ppRE + distance_ppD + distance_LppRE + maximum_ppE
            energies['U-COMB']  += maximum_comb
            minimum_comb           = minimum_ppRE + distance_ppD + distance_LppRE + minimum_ppE
            energies['L-COMB']  += minimum_comb
            mean_comb              = mean_ppRE + distance_ppD + distance_LppRE + mean_ppE
            energies['M-COMB']  += mean_comb

        return energies

    ##################################
    # CONTACT RANDOMIZATION  METHODS #
    ##################################

    def randomize_residue_interactions(self, struct_residues, res_interactions):

        rand_interactions = []
        done = {}
        for res_interaction in res_interactions:
            dist = res_interaction[2]
            if done.has_key(res_interaction[0]):
                res1 = done[res_interaction[0]]
            else:
                res1 = random.choice(struct_residues)
                done[res_interaction[0]] = res1
            if done.has_key(res_interaction[1]):
                res2 = done[res_interaction[1]]
            else:
                res2 = random.choice(struct_residues)
                done[res_interaction[1]] = res2
            rand_interactions.append((res1, res2, dist))
        return rand_interactions

    def compute_Zscores(self, real_energies, rand_energies_lists):

        Zenergies = {} 
        for potential_type in real_energies:
            if potential_type == 'D-3D' or potential_type == 'C-3D':
                continue
            energies_mean = numpy.mean(rand_energies_lists[potential_type])
            energies_std  = numpy.std(rand_energies_lists[potential_type])
            Zenergies[potential_type] = (real_energies[potential_type]-energies_mean)/energies_std
        return Zenergies

    ############################
    # SPLIT POTENTIALS GETTERS #
    ############################

    def get_ppR(self, res1, res2, distance):

        distance = int(math.floor(distance))
        if self.ppR.has_key(res1) and self.ppR[res1].has_key(res2):
            return float(self.ppR[res1][res2][distance])
        elif self.ppR.has_key(res2) and self.ppR[res2].has_key(res1):
            return float(self.ppR[res2][res1][distance])
        else:
            return 0
            #raise RuntimeError('ERROR: Matrix entry not found: %s, %s' % (res1, res2))
        
    def get_ppR_stat(self, res1, res2, stat):

        if self.ppR.has_key(res1) and self.ppR[res1].has_key(res2):
            data = numpy.array(self.ppR[res1][res2], dtype='float64')
        elif self.ppR.has_key(res2) and self.ppR[res2].has_key(res1):
            data = numpy.array(self.ppR[res2][res1], dtype='float64')
        else:
            return 0
            #raise RuntimeError('ERROR: Matrix entry not found: %s, %s, %d' % (res1, res2))
        if stat.upper() == 'UPPER':
            return numpy.max(data)
        elif stat.upper() == 'LOWER':
            return numpy.min(data)
        elif stat.upper() == 'MEAN':
            return numpy.mean(data)
        
    def get_ppE(self, res1, res2, distance):

        distance = int(math.floor(distance))
        if self.ppE.has_key(res1) and self.ppE[res1].has_key(res2):
            return -float(self.ppE[res1][res2][distance])
        elif self.ppE.has_key(res2) and self.ppE[res2].has_key(res1):
            return -float(self.ppE[res2][res1][distance])
        else:
            return 0
            #raise RuntimeError('ERROR: Matrix entry not found: %s, %s' % (res1, res2))

    def get_ppE_stat(self, res1, res2, stat):

        if self.ppE.has_key(res1) and self.ppE[res1].has_key(res2):
            data = numpy.array(self.ppE[res1][res2], dtype='float64')
        elif self.ppE.has_key(res2) and self.ppE[res2].has_key(res1):
            data = numpy.array(self.ppE[res2][res1], dtype='float64')
        else:
            return 0
            #raise RuntimeError('ERROR: Matrix entry not found: %s, %s' % (res1, res2))
        # UPPER and LOWER are inverted in this potential
        if stat.upper() == 'UPPER':
            return -numpy.min(data)
        elif stat.upper() == 'LOWER':
            return -numpy.max(data)
        elif stat.upper() == 'MEAN':
            return -numpy.mean(data)
              
    def get_LppRE(self, res):

        return -float(self.LppRE[res])
    
    def get_ppRE(self, res1, res2, distance):
        '''
        '''
        distance = int(math.floor(distance))
        if self.ppRE.has_key(res1) and self.ppRE[res1].has_key(res2):
            return float(self.ppRE[res1][res2][distance])
        elif self.ppRE.has_key(res2) and self.ppRE[res2].has_key(res1):
            return float(self.ppRE[res2][res1][distance])
        else:
            return 0
            #raise RuntimeError('ERROR: Matrix entry not found: %s, %s' % (res1, res2))

    def get_ppRE_stat(self, res1, res2, stat):
        '''
        '''
        if self.ppRE.has_key(res1) and self.ppRE[res1].has_key(res2):
            data = numpy.array(self.ppRE[res1][res2], dtype='float64')
        elif self.ppRE.has_key(res2) and self.ppRE[res2].has_key(res1):
            data = numpy.array(self.ppRE[res2][res1], dtype='float64')
        else:
            return 0
            #raise RuntimeError('ERROR: Matrix entry not found: %s, %s' % (res1, res2))
        if stat.upper() == 'UPPER':
            return numpy.max(data)
        elif stat.upper() == 'LOWER':
            return numpy.min(data)
        elif stat.upper() == 'MEAN':
            return numpy.mean(data)
        
    def get_ppD(self, distance):
        '''
        '''
        distance = str(int(math.floor(distance)))
        return float(self.ppD[distance])

    #########################################################
    # METHODS TO READ AND PROCESS THE SPLIT POTENTIALS FILE #
    #########################################################

    def read_sp_file(self):

        tag = 0
        sp_file_fo = open(self.sp_file, 'r')
        for line in sp_file_fo:
            if line.startswith("PAIR-POTENTIALS OF RESIDUES AND ENVIRONMENTS"):             tag = 4
            elif line.startswith("PAIR-POTENTIALS OF ENVIRONMENTS"):                        tag = 2
            elif line.startswith("LOCAL PAIR-POTENTIALS OF RESIDUES AND ENVIRONMENTS"):     tag = 3
            elif line.startswith("PAIR-POTENTIALS OF RESIDUES"):                            tag = 1
            elif line.startswith("STANDARD PAIR-POTENTIALS OF DISTANCES"):                  tag = 5
            elif line.startswith("END"):                                                    tag = 0
            self.process_sp_file_line(line, tag)
        sp_file_fo.close()

    def process_sp_file_line(self, line, tag):

        if tag == 0:    
            return
        if tag == 1 and line.startswith(" PAIR"): 
            self.assign_multiple_distances(self.ppR, line.split()[1], line.split()[2:])
        if tag == 2 and line.startswith(" PAIR"): 
            self.assign_multiple_distances(self.ppE, line.split()[1], line.split()[2:])
        if tag == 3 and line.startswith(" AA-ENV"): 
            self.assign_single_distance(self.LppRE, line.split()[1], line.split()[2])
        if tag == 4 and line.startswith(" PAIR"): 
            self.assign_multiple_distances(self.ppRE, line.split()[1], line.split()[2:])
        if tag == 5 and line.startswith(" Distance"): 
            self.assign_single_distance(self.ppD, line.split()[1], line.split()[2])

    def assign_multiple_distances(self, matrix, keys, values):
        
        matrix.setdefault(keys.split(':')[0],{}).setdefault(keys.split(':')[1],[]).extend(values)
        
    def assign_single_distance(self, matrix, key, value):
        
        matrix[key] = value

    ##################################################
    # METHODS TO PRINT THE SPLIT POTENTIALS MATRICES #
    ##################################################

    def two_keys_matrix_2_string_array(self, matrix, prev):
        
        lines = []
        for res1 in matrix:
            for res2 in matrix[res1]:
                line = prev + "%s:%s\t" % (res1, res2)
                line = line + "\t".join(matrix[res1][res2])
                lines.append(line)
        return lines
                
    def one_key_matrix_2_string_array(self, matrix, prev):
        
        lines = []
        for k in matrix:
            line = prev + "\t".join((k,matrix[k]))
            lines.append(line)
        return lines

    def ppR_2_string_array(self):
        
        lines = []
        lines.append("PAIR-POTENTIALS OF RESIDUES")
        lines.extend(self.two_keys_matrix_2_string_array(self.ppR, " PAIR "))
        lines.append("END")
        return lines
    
    def string_ppR(self): 

        return "\n".join(self.ppR_2_string_array())

    def ppE_2_string_array(self):
        
        lines = []
        lines.append("PAIR-POTENTIALS OF ENVIRONMENTS")
        lines.extend(self.two_keys_matrix_2_string_array(self.ppE, " PAIR "))
        lines.append("END")
        return lines
    
    def string_ppE(self): 

        return "\n".join(self.ppE_2_string_array())
    
    def LppRE_2_string_array(self):
        
        lines = []
        lines.append("LOCAL PAIR-POTENTIALS OF RESIDUES AND ENVIRONMENTS")
        lines.extend(self.one_key_matrix_2_string_array(self.LppRE, " AA-ENV "))
        lines.append("END")
        return lines
    
    def string_LppRE(self): 

        return "\n".join(self.LppRE_2_string_array())

    def ppRE_2_string_array(self):
        
        lines = []
        lines.append("PAIR-POTENTIALS OF RESIDUES AND ENVIRONMENTS")
        lines.extend(self.two_keys_matrix_2_string_array(self.ppRE, " PAIR "))
        lines.append("END")
        return lines
    
    def string_ppRE(self): 

        return "\n".join(self.ppRE_2_string_array())

    def ppD_2_string_array(self):
        
        lines = []
        lines.append("STANDARD PAIR-POTENTIALS OF DISTANCES")
        lines.extend(self.one_key_matrix_2_string_array(self.ppD, " Distance "))
        lines.append("END")
        return lines
    
    def string_ppD(self): 

        return "\n".join(self.ppD_2_string_array())

    def __str__(self):
        
        lines = []
        lines.extend(self.ppR_2string_array())
        lines.extend(self.ppE_2string_array())
        lines.extend(self.LppRE_2string_array())
        lines.extend(self.ppRE_2string_array())
        lines.extend(self.ppD_2string_array())
        return "\n".join(lines)
        


import os,sys,re
import argparse
import ConfigParser
import shutil
import string
import numpy

# Add "." to sys.path #
src_path =  os.path.abspath("/home/boliva/PROGRAMS/Interactome/scripts/")
sys.path.append(src_path)

# Read configuration file #
config = ConfigParser.ConfigParser()
config_file = os.path.join(src_path, "config.ini")
config.read(config_file)

# Read SBI path
sbi_path = os.path.join(src_path, config.get("Paths", "sbi_path"))
sys.path.append(sbi_path)
from SBI.structure import *


def fileExist (file):
 if file is not None:
  return os.path.exists(file) and os.path.isfile(file)
 else:
  return False



def printverbose(f,flag,message):
 """Define verbose to print in file 'f', if 'flag'=True, message given as 'message'"""
 if flag: f.write("%s"%(message))


def split(pdb_file,out):
 """
  Split PDB in chains: it creates the PDB and the FastA sequences of each chain and returns the set of pairs (name,sequence)
 """
 if fileExist(pdb_file):
   pdb=PDB(pdb_file)
 else:
  sys.stderr.write("Missing PDB file %s \n"%pdb_file)
  return None
 sequences_split=set()
 for code in pdb.chain_identifiers:
   chain=pdb.get_chain_by_id(code)
   pdb_new=PDB()
   pdb_new.add_chain(chain)
   new_name=out+code+".pdb"
   new_fasta=out+code+".fa"
   name=out.split("/")[-1]+code
   sequence=None
   if chain.chaintype == "N": sequence=chain.nucleotide_sequence()
   if chain.chaintype == "P": sequence=chain.protein_sequence
   #pdb_new.clean
   #pdb_new.write(new_name,force=True)
   #fasta=open(new_fasta,"w")
   #fasta.write(">%s\n%s\n"%(name,sequence))
   #fasta.close()
   if sequence is not None: sequences_split.add((name,sequence))

 return sequences_split
 
def parse_skempi(skempi_file):

   if fileExist(skempi_file):
      fd=open(skempi_file)
   else:
      sys.stderr.write("Missing SKEMPI CSV file %s \n"%skempi_file)
      return None
   ddG_skempi  = {}
   for line in fd:
     if line.startswith('"#'): continue
     data      = line.split('\t')
     try:
      pdb_chain = data[0].lstrip('"').rstrip('"')
      pdb_name  = pdb_chain.split("_")[0]
      chain_a   = pdb_chain.split("_")[1]
      chain_b   = pdb_chain.split("_")[2]
      mutation  = data[2].lstrip('"').rstrip('"')
      Kd_mut    = float(data[7].lstrip('"').rstrip('"'))
      Kd_wt     = float(data[9].lstrip('"').rstrip('"'))
      temp      = float("".join(data[13].lstrip('"').rstrip('"').split()[0][0:2]))
      ddG       =  (8.314/4184.0) * (273.15 + temp) * numpy.log(Kd_mut) - (8.314/4184.0) * (273.15 + temp) * numpy.log(Kd_wt)
      set_a    = set()
      set_b    = set()
      for mutant in mutation.split(","):
           aa_wt    = mutant[0]
           chain    = mutant[1]
           position = mutant[2:-1]
           aa_mut   = mutant[-1]
           if chain == chain_a:
              set_a.add(aa_wt + position + aa_mut)
           if chain == chain_b:
              set_b.add(aa_wt + position + aa_mut)
      form_a="+".join([x for x in set_a])
      form_b="+".join([x for x in set_b])
      pair_wt = pdb_name + "+" + chain_a + "::" + pdb_name + "+" + chain_b
      if len(set_a)>0:
         pair_mut= pdb_name + "+" + chain_a + "_" + form_a + "::"
      else:
         pair_mut= pdb_name + "+" + chain_a  + "::"
      if len(set_b)>0:
         pair_mut= pair_mut + pdb_name + "+" + chain_b + "_" + form_b
      else:
         pair_mut= pair_mut + pdb_name + "+" + chain_b
      ddG_skempi.setdefault(pair_mut,ddG)  
     except Exception as e:
      sys.stderr.write("Error %s. Skip %s %s\n"%(e,pdb_name,mutation))
   return ddG_skempi


def parse_user_arguments(*args, **kwds):

    parser = argparse.ArgumentParser(
        description = "Parse SKEMPI and create MODPIN files",
        epilog      = "@oliva's lab 2018")
    parser.add_argument('-i','--skempi_file',dest='skempi',action = 'store',
                        help = 'SKEMPI Input CSV table')
    parser.add_argument('-p','--pdb_folder',dest='pdb',action = 'store',
                        help = 'PDB directory (i.e. downloaded from SKEMPI)')
    parser.add_argument('-o','--output_file',dest='out',action = 'store', default='output',
                        help = 'Output rootname for files (default is output)')
    parser.add_argument('-m','--max_mutations',dest='maxmut',action='store', default=None,
                        help = 'Maximum number of mutations per chain')
    parser.add_argument('-v','--verbose',dest='verbose',action = 'store_true',
                        help = 'Verbose execution ')


    options=parser.parse_args()

    return options

def main():

 options   = parse_user_arguments()
 if options.verbose: sys.stdout.write("Parsing Skempi\n")
 ddG       = parse_skempi(options.skempi)
 ppi_file  = options.out + ".ppi"
 seq_file  = options.out + ".fa"
 ddG_file  = options.out + ".ddG"
 if options.maxmut is not None:
     maxmut=int(options.maxmut)

 interactions=set()
 fasta={}
 fg=open(ddG_file,"w")
 fg.write("#%39s\t%10s\n"%("Mutant","ddG"))

 for pair,binding in ddG.iteritems():
     if options.verbose: sys.stdout.write("Parse %s\n"%(pair))
     a,b  = pair.split("::")
     wt_a = a.split("_")[0]
     wt_b = b.split("_")[0]
     form_a=None
     form_b=None
     if len(a.split("_"))>1: form_a=a.split("_")[1]
     if len(b.split("_"))>1: form_b=b.split("_")[1]
     pdbA = wt_a.split("+")[0]
     pdbB = wt_b.split("+")[0]
     if len(wt_a.split("+"))>1: chain_a = wt_a.split("+")[1]
     if len(wt_b.split("+"))>1: chain_b = wt_b.split("+")[1]
     if len(chain_a)>1 or len(chain_b)>1:
        if options.verbose: sys.stdout.write("\t-- Skip this interaction\n")
        continue
     if form_a is not None and options.maxmut is not None:
       if len(form_a.split("+"))>maxmut:
        if options.verbose: sys.stdout.write("\t-- Skip this interaction\n")
        continue
     if form_b is not None and options.maxmut is not None:
       if len(form_b.split("+"))>maxmut:
        if options.verbose: sys.stdout.write("\t-- Skip this interaction\n")
        continue
     if pdbA == pdbB:
        pdb_file=options.pdb+"/"+pdbA+".pdb"
        out_file=options.pdb+"/"+pdbA+"+"
        if fileExist(pdb_file):
           sequences=split(pdb_file,out_file)
        else:
           sys.stderr.write("Missing PDB file %s \n"%pdb_file)
     else:
        sys.stderr.write("Interaction in diferent PDB files %s vs %s\n"%(pdbA,pdbB))
     interactions.add(wt_a+"::"+wt_b)
     interactions.add(pair)
     fg.write("%40s\t%10.5f\n"%(pair,binding))
     for (name,seq) in  sequences:
        if name == wt_a:
           fasta.setdefault(name,seq)
           if form_a is not None:
              for mutant in form_a.split("+"):
                  aa_wt    = mutant[0]
                  position = int(mutant[1:-1])
                  aa_mut   = mutant[-1]
                  if seq[position-1] == aa_wt:
                     seq_new=seq[0:position-1] + aa_mut +seq[position:]
                     fasta.setdefault(a,seq_new)
                  else:
                     if options.verbose: sys.stderr.write("Wrong substitution %s in %s\n"%(mutant,a))

        if name == wt_b:
           fasta.setdefault(name,seq)
           if form_b is not None:
              for mutant in form_b.split("+"):
                  aa_wt    = mutant[0]
                  position = int(mutant[1:-1])
                  aa_mut   = mutant[-1]
                  if seq[position-1] == aa_wt:
                     seq_new=seq[0:position-1] + aa_mut +seq[position:]
                     fasta.setdefault(b,seq_new)
                  else:
                     if options.verbose: sys.stderr.write("Wrong substitution %s in %s\n"%(mutant,b))
     
 fg.close()

 fa=open(seq_file,"w")
 for name,seq in fasta.iteritems():
   fa.write(">%s\n%s\n"%(name,seq))
 fa.close()
 fi=open(ppi_file,"w")
 for ppi in interactions:
   a,b=ppi.split("::")
   fi.write("%s\t%s\n"%(a,b))
 fi.close()


if  __name__ == "__main__":
    main()





import re


class IndexedNum(object):
    '''
    {IndexedNum} has been created for PDB.
    Some PDB files contain residues in positions that are not only numeric, but also have an index.
    IndexedNum tries to paliate this by giving the properties of a number (as much as possible) to this kind of number-index combination.
    Thus, a 7 becomes a '7 ' but can still be used as a number.
    Comparissons, addition and int(), float() casts are implemented
    '''
    number_regex = re.compile('(\-*\d+)(\w*)')

    def __init__(self, value):
        '''
        {IndexedNum} has two simple attributes:
            - number (int)
            - index  (string)
        If it is initilized with X, it means that we have a "blank" position.
        To avoid mixups, the index is transformed to '_X_'

        @type  value: String
        @type  value: IndexedNum
        @type  value: int
        @param value: data to transform to a indexed number
        '''
        if isinstance(value, IndexedNum):
            self._num = value._num
            self._idx = value._idx
        else:
            value = str(value)
            if value != 'X':
                parts = re.search(self.number_regex, value)
                self._num = int(parts.group(1))
                self._idx = parts.group(2)
                if self._idx == '':
                    self._idx = ' '
            else:
                self._num = 0
                self._idx = '_X_'

    #
    # ATTRIBUTES
    #
    @property
    def number(self):
        return self._num

    @property
    def index(self):
        return self._idx

    #
    # BOOLEANS
    #
    @property
    def is_integer(self):
        return self._idx == ' '

    @property
    def is_blank(self):
        return self._idx == '_X_'

    #
    # DEFAULT COMPARATIVE FUNCTIONS
    #

    def __lt__(self, other):
        if isinstance(other, int):
            return self.number < other
        elif isinstance(other, IndexedNum):
            if self.number < other.number:
                return True
            elif self.number == other.number:
                return self.index < other.index
            else:
                return False
        return NotImplemented

    def __le__(self, other):
        if isinstance(other, int):
            return self.number <= other
        elif isinstance(other, IndexedNum):
            if self.number < other.number:
                return True
            elif self.number == other.number:
                return self.index <= other.index
            else:
                return False
        return NotImplemented

    def __eq__(self, other):
        if isinstance(other, int):
            return self.number == other
        elif isinstance(other, IndexedNum):
            return self.number == other.number and self.index == other.index
        elif isinstance(other, basestring):
            return str(self).strip() == other.strip()
        return NotImplemented

    def __ne__(self, other):
        result = self.__eq__(other)
        if result is NotImplemented:
            return result
        return not result

    def __gt__(self, other):
        if isinstance(other, int):
            return self.number > other
        elif isinstance(other, IndexedNum):
            if self.number > other.number:
                return True
            elif self.number == other.number:
                return self.index > other.index
            else:
                return False
        return NotImplemented

    def __ge__(self, other):
        if isinstance(other, int):
            return self.number >= other
        elif isinstance(other, IndexedNum):
            if self.number > other.number:
                return True
            elif self.number == other.number:
                return self.index >= other.index
            else:
                return False
        return NotImplemented

    def __add__(self, other):
        if isinstance(other, int):
            return IndexedNum(str(self.number + other) + self.index)
        if isinstance(other, IndexedNum) and other.is_integer:
            return IndexedNum(str(self.number + other) + self.index)
        return NotImplemented

    def __iadd__(self, other):
        if isinstance(other, int):
            self.number += other
        if isinstance(other, IndexedNum) and other.is_integer:
            self.number += other
        return NotImplemented

    def __int__(self):
        return int(self.number)

    def __float__(self):
        return float(self.number)

    def __hash__(self):
        return repr(self)

    def __repr__(self):
        if self.is_blank:
            return 'X'
        else:
            return (str(self.number) + self.index).strip()

    def __str__(self):
        return repr(self)



from SBI.data       import aminoacids3to1, nucleic3to1
from .chain         import Chain, ChainOfProtein, ChainOfNucleotide
from .header.Header import PDBHeader
import re


def read_PDB_header(pdbobject):

    PDBH = PDBHeader()
    for line in pdbobject.pdb_file.descriptor:
        line = re.sub('\'', '\\\'', line)   # JIC
        # CONTROL FOR AFTERWARDS
        if line.startswith('HEADER'):       # BASIC INFORMATION
            PDBH.add_header(line)
        elif line.startswith('TITLE'):      # EXPERIMENT TITLE (ML)
            PDBH.add_title(line)
        elif line.startswith('EXPDTA'):     # EXPERIMENT TYPE (ML)
            PDBH.add_experiment(line, 'type')
        elif line.startswith('REMARK   2 RESOLUTION'):
            PDBH.add_experiment(line, 'resolution')
        elif line.startswith('REMARK   3   R VALUE'):
            PDBH.add_experiment(line, 'rfactor')
        elif line.startswith('REMARK   3   FREE R VALUE     '):
            PDBH.add_experiment(line, 'freeR')
        elif line.startswith('SPRSDE'):
            PDBH.add_deprecated(line)
        elif line.startswith('COMPND'):     # MOLECULE PARSING
            PDBH.add_molecule(line, 'COMPND', re.search(r'MOL_ID\:', line))
        elif line.startswith('SOURCE'):
            PDBH.add_molecule(line, 'SOURCE',  re.search(r'MOL_ID\:', line))
        elif line.startswith('KEYWDS'):     # KEYWORDS (ML)
            PDBH.add_keywords(line)
        elif line.startswith('DBREF '):     # REFERENCES TO OTHER DB
            PDBH.add_dbreference(line)
        elif line.startswith('REMARK 800 SITE_IDENTIFIER'):  # SITES
            PDBH.add_site(line, 'IDENTIFIER')
        elif line.startswith('REMARK 800'):
            PDBH.add_site(line, 'REMARK')
        elif line.startswith('SITE'):
            PDBH.add_site(line, 'SITE')
        elif line.startswith('HET   '):     # HETEROATOMS (ML)
            PDBH.add_hetatom(line, 'HET')
        elif line.startswith('HETNAM'):
            PDBH.add_hetatom(line, 'HETNAM')
        elif line.startswith('FORMUL'):
            PDBH.add_hetatom(line, 'FORMUL')
        elif line.startswith('HELIX '):     # SECONDARY STRUCTURE
            PDBH.add_secondary_structure(line, 'HELIX')
        elif line.startswith('SHEET '):
            PDBH.add_secondary_structure(line, 'SHEET')
        elif line.startswith('TURN '):
            PDBH.add_secondary_structure(line, 'TURN')
        elif line.startswith('REMARK 290   SMTRY'):  # MATRICES (ML)
            PDBH.add_simetry_matrix(line)
        elif line.startswith('REMARK 350 BIOMOLECULE'):
            PDBH.add_biomolecule(line)
        elif line.startswith('REMARK 350 APPLY THE FOLLOWING TO CHAINS'):
            PDBH.link_biomolecule(line)
        elif line.startswith('REMARK 350                    AND CHAINS'):
            PDBH.link_biomolecule(line)
        elif line.startswith('REMARK 350   BIOMT'):
            PDBH.add_biomolecule_matrix(line)
        elif line.startswith('ATOM') or line.startswith('HETATM'):
            break

    PDBH.process()

    pdbobject._header = PDBH


def read_PDB_file(pdbobject, biomolecule=False):
    """
    Process and load crystal data from a PDB formated file
    """
    read = True

    #Open takes into account the possibility of a gzipped file
    pdb_fd = pdbobject.pdb_file.descriptor

    #To force the creation of a new structure
    old_chain = "OLD_CHAIN"

    for line in pdb_fd:
        #In NMR, this will force to read only one model
        if line.startswith("ENDMDL"):
            old_chain = "OLD_CHAIN"
            pdbobject._NMR = True

        if line.startswith("ATOM") or line.startswith("HETATM"):
            chain = line[21:22].strip()
            if chain != old_chain:
                #The PDB assigned to the structure is the filename WITHOUT THE PATH
                if pdbobject.is_NMR or (not pdbobject.chain_exists(chain) or biomolecule):
                    chain_type = line[17:20].strip()
                    if aminoacids3to1.has_key(chain_type):
                        obj_chain = ChainOfProtein(pdb = pdbobject.pdb_file.prefix ,chain = chain)
                        pdbobject._has_prot = True
                    elif nucleic3to1.has_key(chain_type):
                        obj_chain = ChainOfNucleotide(pdb = pdbobject.pdb_file.prefix ,chain = chain)
                        pdbobject._has_nucl = True
                    else:
                        obj_chain = Chain(pdb = pdbobject.pdb_file.prefix ,chain = chain)
                    pdbobject.add_chain(obj_chain, NMR = pdbobject.is_NMR)
                else:
                    putative_old_chain = pdbobject.get_chain_by_id(id = chain)
                    obj_chain = putative_old_chain

                old_chain = chain
                read = True

            if read:
                chain_type = line[17:20].strip()
                if not isinstance(obj_chain, ChainOfNucleotide) and not isinstance(obj_chain, ChainOfProtein):
                    if nucleic3to1.has_key(chain_type):
                        newobj_chain = ChainOfNucleotide(pdb = pdbobject.pdb_file.prefix ,chain = obj_chain.chain)
                        pdbobject._has_nucl = True
                        newobj_chain.fuse(chain = obj_chain, lapl = True)
                        del(obj_chain)
                        obj_chain = newobj_chain
                        pdbobject._chains[pdbobject._get_chain_position_by_id(id = obj_chain.chain)] = obj_chain
                    if aminoacids3to1.has_key(chain_type):
                        newobj_chain = ChainOfProtein(pdb = pdbobject.pdb_file.prefix ,chain = obj_chain.chain)
                        pdbobject._has_prot = True
                        newobj_chain.fuse(chain = obj_chain, lapl = True)
                        del(obj_chain)
                        obj_chain = newobj_chain
                        pdbobject._chains[pdbobject._get_chain_position_by_id(id = obj_chain.chain)] = obj_chain

                obj_chain.read_PDB_line(line)

        if line.startswith("TER"):
            obj_chain.is_term
            old_chain = "OLD_CHAIN"

    pdbobject.pdb_file.close()

    pdbobject._chain_id = set([c.chain for c in pdbobject.chains if not c.is_empty])
    pdbobject._chains   = [c for c in pdbobject.chains if not c.is_empty]

def read_PDB_line(thischain, line, keep_version = "A"):
    """
    Given a PDB-formated line, creates an atom to add to a new or a pre-existent residue

    @type  line: String
    @param line: PDB formated line

    @type  keep_version: String
    @param keep_version: Some residues have two versions, codified in front of the residue_type (line[16:17])
                         By default we keep the A version of doubles Aa, but it can be changed through parameters
    """
    isOK = set([" ", keep_version, thischain._residue_version])

    #This skips several versions of the same residue. It always will grab " " and keep_version
    #but in chains that are all B (1EA4_Z) this will help grab them
    if line[16:17] not in isOK:
        if len(thischain) == 0:
            thischain._residue_version = line[16:17]
        else:
            return

    residue_num = int(line[22:26].strip())
    residue_ver = line[26:27]

    #Check the need of creating a new residue
    if thischain.is_empty or (len(thischain._last_appended_list) > 0 and thischain._last_appended_list[-1].identifier != str(residue_num) + residue_ver):
        residue = thischain._new_Residue(number = residue_num, version = residue_ver, Rtype = line[17:20].strip(), mode = line[:6].strip())
        thischain.add_residue(residue)


    #Create the new atom
    x, y, z = [float(line[30+8*i:38+8*i]) for i in range(3)]
    try:
        occupancy = float(line[54:60])
    except:
        occupancy = ""
    try:
        tempFactor = float(line[60:66])
    except:
        tempFactor = ""
    try:
        element = line[76:78].strip()
    except:
        element = ""
    try:
        charge = line[78:80].strip()
    except:
        charge = ""
    atom = thischain._new_Atom(number = line[6:12].strip(), name = line[12:16].strip(), x = x, y = y, z = z, occupancy = occupancy, tempFactor = tempFactor, element = element, charge = charge)

    #Add the atom to the last residue (it applies the same if the residue has been created now or before)
    thischain._last_appended_list[-1].add_atom(atom)

# Abbreviations:
# --------------
# (ML) : MultiLine



import sys
import argparse
import os
import re
import ConfigParser
import shutil
import subprocess
import gzip

# Add '.' to sys.path
src_path = os.path.abspath(os.path.dirname(__file__))+"/../../scripts"
sys.path.append(src_path)

# Read configuration file
config = ConfigParser.ConfigParser()
config_file = os.path.join(src_path, 'config.ini')
config.read(config_file)

# Add SBI library path to sys.path
src_path = os.path.join(config.get('Paths','modppi_path'),config.get('Paths', 'sbi_library_path'))
sys.path.append(src_path)

# Add functions path to sys.path
src_path = os.path.join( config.get('Paths','modppi_path'), config.get('Paths', 'functions_path') )
sys.path.append(src_path)

# Import functions

from SBI.structure import PDB
import SBI.structure.contacts as cn
from functions import *
from SeqIO import *


def main():
   #Initialize
   options=parse_user_arguments()
   verbose=options.show
   pdb_path=os.path.join( config.get('Paths','modppi_path'),config.get('Paths', 'pdb_path') )
   try:
    did_path=os.path.join(config.get('Paths','modppi_path'),config.get('Paths','3did_path'))
    data_path=os.path.join(config.get('Paths','modppi_path'),config.get('Paths','data_path'))
   except:
    did_path=options.outdir
    data_path=options.outdir

   if not os.path.exists(did_path):
    sys.stderr.write("No 3DID directory, please check your installation or INPUT\n")
    
   if not os.path.exists(data_path):
    sys.stderr.write("No DATA directory, please check your installation or INPUT\n")
     

   #Create list of interactions and FASTA sequences of 3DiD
   did_interactions=open(os.path.join(data_path,options.interactions_file),"w")
   did_fasta       =open(os.path.join(data_path,options.seq_file),"w")
   for brk in os.listdir(did_path):
       if verbose: sys.stderr.write("\t\t-- Reading %s  \n"%os.path.join(did_path,brk))
       try:
          pdb=PDB(os.path.join(did_path,brk))
          id_chain=[]
          for c in pdb.chain_identifiers:
           pdb_chain=pdb.get_chain_by_id(c)
           id_chain.append(pdb.id+"_"+c)
           printfasta( did_fasta, pdb.id+"_"+c , pdb_chain.gapped_protein_sequence)      
          did_interactions.write("%s\t%s\n"%(id_chain[0],id_chain[1]))
       except Exception as e:
          if verbose: sys.stderr.write("\t\t-- %s cannot be read\n\t\t   Error: %s\n"%(os.path.join(did_path,brk),e))
          continue
   did_interactions.close()
   did_fasta.close()


def parse_user_arguments(*args, **kwds):
    parser = argparse.ArgumentParser(
        description = 'Generate the gapped sequences and list of DDIs with the 3DiD structures',
        epilog      = '@Oliva\'s lab 2016')
    parser.add_argument('-ppi', '--interactions', dest = 'interactions_file', action = 'store',
                        help = 'Output file with a list of the interactions from 3did (default is 3did.ddi)')
    parser.add_argument('-seq', '--fasta', dest = 'seq_file', action = 'store',
                        help = 'Output file with FASTA sequences of the proteins from 3DiD (default is 3did_gapped.fasta) ')
    parser.add_argument('-v', '--verbose', dest = 'show', action = 'store_true',
                        help = 'Flag for verbose mode (default is False)')
    options = parser.parse_args()
    return options



if __name__ == '__main__':
    main()





import datetime

from . import process_HEADER_line
from . import process_OBSLTE_line
from . import process_TITLE_line
from . import process_SUPERSEEDED_line
from . import process_MOLKEY
from . import process_KEYWRD_line
from . import process_SITE_IDENTIFIER_line
from . import process_REMARK800_line
from . import process_SITE_lines
from . import process_HETEROATOM_lines
from . import process_SECONDARYSTRUCTURE_lines
from . import process_MATRIX_lines
from . import process_REMARK350_lines
from . import process_SEQRES_lines

from .Experiment         import Experiment
from .Molecule           import Molecule
from .DBreference        import DBref
from .Site               import Site
from .HeteroAtom         import Hetero
from .SecondaryStructure import HelixInfo
from .SecondaryStructure import SheetInfo
from .SecondaryStructure import TurnInfo
from .BioMolecule        import BioMolecule
from SBI.beans.JSONer    import JSONer


class PDBHeader(JSONer):
    # TODO: search hetero by name

    def __init__(self):

        self._pdb        = None     # PDB ID
        self._date       = None     # Deposition Date
        self._obslte     = None     # Retirement Date
        self._replaces   = []       # PDB that replaced this
        self._header     = None     # Classification
        self._title      = None     # Experiment Tilte
        self._keywords   = ''       # Words of interest

        self._experiment = None     # Crystalographic Data
        self._deprecated = []       # Previous PDBs substituted by this one

        self._chaindict  = {}       # Dictionary of chains
                                    # Used to access to data by chain
        self._workchain  = None     # Working chain (full private)

        self._molecules  = {}       # Molecules in the PDB
        self._sites      = {}       # Sites of the PDB
        self._hetero     = {}       # Heteroatoms in the PDB
        self._dbrefs     = []       # References to other DB

        self._secstr     = []       # Secondary Structures

        self._symmetryM  = BioMolecule(0)  # Symmetry Matrices
        self._biomolecM  = []              # BioMolecule Matrices

    #
    # ATTRIBUTES
    #
    @property
    def pdb(self):
        return self._pdb if self._pdb is not None else ''

    @property
    def date(self):
        return self._date if self._date is not None else ''

    @property
    def date_as_date(self):
        if self._date is None:
            return None
        return datetime.date(*[int(x) for x in self.date.split('-')])

    @property
    def obsolete_date(self):
        return self._obslte if self._obslte is not None else ''

    @property
    def obsolete_date_as_date(self):
        if self._obslte is None:
            return None
        return datetime.date(*[int(x) for x in self.obsolete_date.split('-')])

    @property
    def replaced(self):
        return self._replaces

    @property
    def header(self):
        return self._header if self._header is not None else ''

    @property
    def title(self):
        return self._title if self._title is not None else ''

    @property
    def keywords(self):
        return self._keywords

    @property
    def experiment(self):
        return self._experiment

    @property
    def xpdta(self):
        return self.experiment.xpdta

    @property
    def resolution(self):
        return self.experiment.resolution

    @property
    def rfactor(self):
        return self.experiment.rfactor

    @property
    def freeR(self):
        return self.experiment.freeR

    @property
    def deprecated(self):
        return self._deprecated

    @property
    def molecules(self):
        return self._molecules

    @property
    def chains(self):
        return self._chaindict.keys()

    @property
    def sites(self):
        return self._sites

    @property
    def hetero(self):
        return self._hetero

    @property
    def dbrefs(self):
        return self._dbrefs

    @property
    def secondary_structures(self):
        return self._secstr

    @property
    def symmetry_matrix(self):
        return self._symmetryM

    @property
    def biomolecules(self):
        return self._biomolecM

    @property
    def sequences(self):
        data = {}
        for chain in self._chaindict:
            if 'SEQ' in self._chaindict[chain]:
                data[chain] = self._chaindict[chain]['SEQ']
        return data

    #
    # BOOLEANS
    #
    @property
    def are_molecules_processed(self):
        if len(self._molecules) > 0:
            k = self._molecules.keys()
            return self._molecules[k[0]].is_processed
        else:
            return True  # To avoid call the function

    #
    # READ FUNCTIONS
    #
    def add_header(self, line):
        data = process_HEADER_line(line)
        self._pdb    = data[2]
        self._header = data[0]
        self._date   = data[1]

    def add_obslte(self, line):
        data = process_OBSLTE_line(line)
        if data[0] != '':
            self._obslte = data[0]
        for x in data[1:]:
            if x.upper() != 'NONE':
                self._replaces.append(x)

    def add_title(self, line):
        data = process_TITLE_line(line)
        if self._title is None:
            self._title = data
        else:
            self._title += ' ' + data

    def add_experiment(self, line, switch):
        if switch == 'type':
            if self._experiment is None:
                self._experiment = Experiment(line)
            else:
                self._experiment.update_experiment(line)
        elif switch == 'resolution':
            self._experiment.add_resolution(line)
        elif switch == 'rfactor':
            self._experiment.add_rfactor(line)
        elif switch == 'freeR':
            self._experiment.add_freer(line)

    def add_deprecated(self, line):
        self._deprecated.extend(process_SUPERSEEDED_line(line))

    def add_molecule(self, line, switch, initkey):
        if initkey:
            self._workchain = process_MOLKEY(line)
            if switch == 'COMPND':
                self._molecules[self._workchain] = Molecule(self.pdb)
            elif switch == 'SOURCE':
                now_molecule = self._molecules[self._workchain]
                now_molecule._parse_cmpnd()
                for chain in now_molecule.chains:
                    self._chaindict.setdefault(chain, {'MOL': now_molecule})
        else:
            if self._workchain is None:  # Deprecated PDBs like 1RS3
                self._workchain = 1
                self._molecules[self._workchain] = Molecule(self.pdb)
            else:
                self._molecules[self._workchain].add_line(switch, line)

    def add_keywords(self, line):
        self._keywords += process_KEYWRD_line(line)

    def add_dbreference(self, line):
        self._dbrefs.append(DBref(line))
        chain = self.dbrefs[-1].chain
        if chain in self._chaindict:
            self._chaindict[chain].setdefault('DBREF', [])
        else:
            self._chaindict.setdefault(chain, {'DBREF': []})
        self._chaindict[chain]['DBREF'].append(self.dbrefs[-1])

    def add_site(self, line, switch):
        if switch == 'IDENTIFIER':
            self._workchain = process_SITE_IDENTIFIER_line(line)
            self._sites[self._workchain] = Site(self._workchain)
        elif switch == 'REMARK':
            data = process_REMARK800_line(line)
            if self._workchain in self._sites:
                self._sites[self._workchain].add_remark(data)
        elif switch == 'SITE':
            data = process_SITE_lines(line)
            self._sites.setdefault(data[0], Site(data[0]))
            self._sites[data[0]].add_residues(data[1:])

    def add_hetatom(self, line, switch):
        data = process_HETEROATOM_lines(line)
        if data[0] != 'HOH':   # SKIP WATERS
            if switch == 'HET':
                self._hetero.setdefault(data[0], Hetero(data[0], data[1]))
            elif switch == 'HETNAM':
                if data[0] in self._hetero:
                    self._hetero[data[0]].add_name(data[1])
            elif switch == 'FORMUL':
                if data[0] in self._hetero:
                    self._hetero[data[0]].add_formula(data[1])

    def add_secondary_structure(self, line, switch):
        data = process_SECONDARYSTRUCTURE_lines(line)
        if switch == 'HELIX':
            self._secstr.append(HelixInfo(data))
        elif switch == 'SHEET':
            self._secstr.append(SheetInfo(data))
        elif switch == 'TURN':
            self._secstr.append(TurnInfo(data))

    def add_simetry_matrix(self, line):
        data = process_MATRIX_lines(line)
        if int(data[0]) == 1:
            self.symmetry_matrix.new_matrix()
        self.symmetry_matrix.update_last_matrix(*data)

    def add_biomolecule(self, lines):
        data = process_REMARK350_lines(lines)
        self._biomolecM.append(BioMolecule(data))

    def link_biomolecule(self, lines):
        data = process_REMARK350_lines(lines)
        data = [x.strip() for x in data.split(',')]
        if len(self._biomolecM) == 0:
            self.add_biomolecule(':1')
        self._biomolecM[-1].chains = data

    def add_biomolecule_matrix(self, lines):
        data = process_MATRIX_lines(lines)
        if int(data[0]) == 1:
            self._biomolecM[-1].new_matrix()
        self._biomolecM[-1].update_last_matrix(*data)

    def add_sequence(self, line):
        data = process_SEQRES_lines(line)
        self._chaindict[data[0]].setdefault('SEQ', '')
        self._chaindict[data[0]]['SEQ'] += data[1]

    #
    # FINAL PROCESSING FUNCTION
    #
    def process(self):
        # Final processing of Molecule data
        if not self.are_molecules_processed:
            for m in self._molecules:
                self._molecules[m]._parse()
        # Processing keywords
        self._keywords = list(set([x.strip() for x in self._keywords.split(',')]))
        if len(self._keywords) == 1 and self._keywords[0] == '':
            self._keywords = []
        # Assigning sites to chain dictionary
        for k, v in self.sites.iteritems():
            for c in v.chains:
                if c not in self._chaindict:
                    if len(self._molecules) > 0:
                        nm = max([int(x) for x in self._molecules.keys()]) + 1
                    else:
                        nm = 1
                    self._molecules[nm] = Molecule(self.pdb)
                    self._chaindict.setdefault(c, {'MOL': self._molecules[nm]})
                    self._chaindict[c]['MOL']._chains.append(c)
                elif 'MOL' not in self._chaindict[c]:
                    self._chaindict[c]['MOL'] = Molecule(self.pdb)
                self._chaindict[c].setdefault('SITES', [])
                self._chaindict[c]['SITES'].append(v)
        # Assigning heteroatoms to chain dictionary
        for k, v in self.hetero.iteritems():
            for c in v.chain:
                if c not in self._chaindict:
                    if len(self._molecules) > 0:
                        nm = max([int(x) for x in self._molecules.keys()]) + 1
                    else:
                        nm = 1
                    self._molecules[nm] = Molecule(self.pdb)
                    self._chaindict.setdefault(c, {'MOL': self._molecules[nm]})
                    self._chaindict[c]['MOL']._chains.append(c)
                self._chaindict[c].setdefault('HETATM', [])
                self._chaindict[c]['HETATM'].append(v)
        # Assigning structures to chain dictionary
        for ss in self.secondary_structures:
            if ss.chain not in self._chaindict:
                if len(self._molecules) > 0:
                        nm = max([int(x) for x in self._molecules.keys()]) + 1
                else:
                    nm = 1
                self._molecules[nm] = Molecule(self.pdb)
                self._chaindict.setdefault(ss.chain, {'MOL': nm})
            self._chaindict[ss.chain].setdefault('SSTRUC', [])
            self._chaindict[ss.chain]['SSTRUC'].append(ss)
        # Assigning all chains to Symmetry Matrix
        self._symmetryM.chains = self._chaindict.keys()

    #
    # FUNCTIONS
    #
    def get_secondary_structures_definition_by(self, chain, start, end):
        ss = []
        for ssdef in self._chaindict[ss.chain]['SSTRUC']:
            if ssdef.defines(chain, start, end):
                ss.append(ssdef)
        return ss

    def get_chainheader(self, chain):
        if chain not in self.chains:
            raise KeyError('No chain {0} in the PDB header'.format(chain))
        return ChainHeader(self._chaindict[chain], self.experiment,
                           self.pdb,               chain)

    def has_hetero(self, heteroID):
        return heteroID in self.hetero.keys()

    def get_hetero_by_name_like(self, hetero_name):
        myheteros = []
        for h in self.hetero:
            if hetero_name.upper() in self.hetero[h].name:
                myheteros.append(self.hetero[h])
        return myheteros

    #
    # OUT FUNCTIONS
    #
    def as_dict(self):
        data = {'chains':    sorted(self.chains),  'date':      self.date,
                'classification': self.header,     'title':     self.title,
                'deprecated':     self.deprecated, 'id':        self.pdb,
                'keywords':       self.keywords,   'molecules': [],
                'experiment':     {},              'dbrefs':    [],
                'sites':          [],              'hetatm':    [],
                'sstruct':        [],              'sequences': self.sequences,
                'symmetry':       self.symmetry_matrix.as_dict(),
                'biomolecules':   [x.as_dict() for x in self.biomolecules],
                'obsolete_date':  self.obsolete_date,
                'replaces':       self.replaced
                }

        for k, x in self.molecules.iteritems():
            data['molecules'].append(x.as_dict())

        if self.experiment is not None:
            data['experiment'] = self.experiment.as_dict()

        if len(self.dbrefs) > 0:
            data['dbrefs']  = [x.as_dict() for x in self.dbrefs]

        if len(self.sites) > 0:
            data['sites']   = [x.as_dict() for k, x in self.sites.iteritems()]

        if len(self.hetero) > 0:
            data['hetatm']  = [x.as_dict() for k, x in self.hetero.iteritems()]

        if len(self.secondary_structures) > 0:
            data['sstruct'] = [x.as_dict() for x in self.secondary_structures]

        return data

    def __repr__(self):
        return self.json(pretty=True)


class ChainHeader(JSONer):
    def __init__(self, datadict, experiment, pdb, chain):
        self._data = datadict
        self._exp  = experiment
        self._pdb  = pdb
        self._chn  = chain

    #
    # ATTRIBUTES
    #
    @property
    def pdb(self):
        return self._pdb if self._pdb is not None else ''

    @property
    def chain(self):
        return self._chn

    @property
    def molecule(self):
        return self._data['MOL']

    @property
    def sequence(self):
        if 'SEQ' in self._data:
            return self._data['SEQ']
        else:
            return ''

    @property
    def sites(self):
        if 'SITES' not in self._data:
            return []
        return self._data['SITES']

    @property
    def dbreferences(self):
        if 'DBREF' not in self._data:
            return []
        return self._data['DBREF']

    @property
    def hetero(self):
        if 'HETATM' not in self._data:
            return []
        return self._data['HETATM']

    @property
    def secondary_structures(self):
        if 'SSTRUC' not in self._data:
            return []
        return self._data['SSTRUC']

    @property
    def xpdta(self):
        return self._exp.xpdta

    @property
    def resolution(self):
        return self._exp.resolution

    @property
    def rfactor(self):
        return self._exp.rfactor

    @property
    def freeR(self):
        return self._exp.freeR

    #
    # FUNCTIONS
    #
    def has_hetero(self, heteroID):
        for h in self.hetero:
            if h.restype == h:
                return True
        return False

    #
    # OUT FUNCTIONS
    #
    def as_dict(self):
        data = {'id':       self.pdb, 'chain':    self.chain,
                'experiment':     {}, 'dbrefs':   [],
                'sites':          [], 'hetatm':   [],
                'secstruct':      [], 'molecule': self.molecule.as_dict()}

        if self._exp is not None:
            data['experiment'] = self._exp.as_dict()

        if len(self.dbreferences) > 0:
            data['dbrefs']     = [x.as_dict() for x in self.dbreferences]

        if len(self.sites) > 0:
            data['sites']      = [x.as_dict() for x in self.sites]

        if len(self.hetero) > 0:
            data['hetatm']     = [x.as_dict() for x in self.hetero]

        if len(self.secondary_structures) > 0:
            for x in self.secondary_structures:
                data['secstruct'].append(x.as_dict())

        return data

    def __repr__(self):
        return self.json(pretty=True)



import sys
from BioLib.Tools.BioExceptions import *

class Residue(object):
    '''
    Represents an structure residue
    '''
    aa_dic   = {'ALA':"A", 'ARG':"R", 'ASN':"N", 'ASP':"D", 'ASX':"B", 'CYS':"C", 'GLN':"Q", 'GLU':"E",
                'GLX':"Z", 'GLY':"G", 'HIS':"H", 'ILE':"I", 'LEU':"L", 'LYS':"K", 'MET':"M", 'PHE':"F",
                'PRO':"P", 'SER':"S", 'THR':"T", 'TRP':"W", 'TYR':"Y", 'VAL':"V", 'SEC':"U", 'PYL':"O", 
                'XLE':"J", 'HSE':"H", 'HSD':"H", 'MSE':"M", 'DVA':"V" }
    surface  = {'A': 115, 'C': 149, 'D': 170, 'E': 207, 'F': 230, 'G': 86,  'H': 206,
                'I': 187, 'K': 222, 'L': 192, 'M': 210, 'N': 184, 'P': 140, 'Q': 208,
                'R': 263, 'S': 140, 'T': 164, 'V': 161, 'W': 269, 'Y': 257 }
    polarity = {'A': False, 'C': False, 'D': True,  'E': True,  'F': False, 'G': False, 'H': True,
                'I': False, 'K': True,  'L': False, 'M': False, 'N': True,  'P': False, 'Q': True,
                'R': True,  'S': True,  'T': True,  'V': False, 'W': False, 'Y': True }
    charge   = {'A': 0, 'C': 0, 'D':-1, 'E':-1, 'F': 0, 'G': 0, 'H': 0,
                'I': 0, 'K': 1, 'L': 0, 'M': 0, 'N': 0, 'P': 0, 'Q': 0,
                'R': 1, 'S': 0, 'T': 0, 'V': 0, 'W': 0, 'Y': 0 }

    def __init__(self, num, res_type):
        '''
        Constructor
        '''
        self.num = int(num)
        self.type = res_type
        self.atoms = []
        self.ss = None
        self.acc = None
        self.ca = None
        self.cb = None

    # Getters #
    def get_num(self):
        return self.num

    def get_type(self):
        return self.type

    def get_atoms(self):
        return self.atoms

    def get_atom_by_num(self, num):
        for atom in self:
            if atom.get_num() == num:
                return atom

    def get_number_atoms(self):
        return len(self.atoms)

    def get_ss(self):
        return self.ss

    def get_acc(self):
        return self.acc

    def get_ca(self):
        return self.ca

    def get_cb(self, glyCA=True):
        if glyCA and self.is_gly():
            return self.ca
        return self.cb

    def get_charge(self):
        try:
            charge = Residue.charge[self.get_type_short()]
        except ResidueTypeShortError as e:
            raise e
        except KeyError as e:
            raise ResidueChargeError(self.get_num())
        return charge

    def get_triad(self):
        '''
        Needed by SplitPotentials library
        '''
        try:
            # Polarity
            if Residue.polarity[self.get_type_short()]: p = 'p'
            else:                                       p = 'n'
            # Exposition
            if self.is_exposed():                       e = 'E'
            else:                                       e = 'B'
        except ResidueTypeShortError as e:
            raise ResidueTriadError(self.get_num(), str(e))
        except KeyError as e:
            raise ResidueTriadError(self.get_num(), 'Cannot get polarity\n')
        except ResidueExpositionError as e:
            raise ResidueTriadError(self.get_num(), str(e))

        if self.get_ss() == None:
            raise ResidueTriadError(self.get_num(), 'Cannot get secondary structure\n')

        return '%s-%s-%s-%s' % (self.get_type_short(), p, self.get_ss(), e)

    # Setters #
    def set_num(self, num):
        '''
        Modify the residue number
        '''
        self.num = num

    def set_ss(self, ss):
        '''
        Sets secondary structure
        '''
        self.ss = ss

    def set_acc(self, acc):
        '''
        Sets accesible surface area
        '''
        self.acc = acc
   
    # Class methods #
    def add_atom(self, atom):
        self.atoms.append(atom)
        if atom.get_type().startswith('CA'): self.ca = atom
        elif atom.get_type().startswith('CB'): self.cb = atom

    def is_exposed(self, threshold=0.6):
        '''
        Returns if the residue is exposed (more than 60 percent exposed by default)
        Baldo orifginal script: int(10*float(self.acc)/Residue.surface[self.get_type_short()]) > 5
        DSSP must be executed before in order to get the acc (accessibility)
        '''
        try:
            return float(self.acc)/Residue.surface[self.get_type_short()] >= threshold
        except TypeError as e:
            raise ResidueExpositionError(self.get_num(), 'No ACC\n')
        except ResidueTypeShortError as e:
            raise ResidueExpositionError(self.get_num(), str(e))
        except KeyError as e:
            raise ResidueExpositionError(self.get_num(), 'Cannot get surface\n')

    def is_gly(self):
        '''
        Return True if the residue is a Glycine, else return False
        '''
        if self.get_type() == 'GLY':
            return True
        return False

    def is_polar(self):
        '''
        Return True if the residue is Polar, else return False
        '''
        try:
            polar = Residue.polarity[self.get_type_short()]
        except ResidueTypeShortError as e:
            raise e
        except KeyError as e:
            raise ResiduePolarityError(self.get_num())
        return polar

    def has_ca(self):
        '''
        Return True if residue has CA, else return False
        '''
        if self.ca is not None:
            return True
        return False

    def has_cb(self):
        '''
        Return True if residue has CB, else return False
        '''
        if self.cb is not None:
            return True
        return False

    def get_type_short(self):
        '''
        Converts the 3 letter type into 1 letter type (example: 'ALA' -> 'A')
        '''
        try:
            return Residue.aa_dic[self.get_type()]
        except KeyError:
            raise ResidueTypeShortError(self.get_type())
    
    def _get_min_distance(self, residue):
        '''
        Gets the minimum distance between two residues
        '''
        try:
            return min([x.get_distance(y) for x in self for y in residue])
        except:
            raise ResidueDistanceError('MIN', self.get_num(), residue.get_num())

    def _get_ca_distance(self, residue):
        '''
        Gets the distance between the CA of two residues
        '''
        try:
            return self.ca.get_distance(residue.ca)
        except:
            raise ResidueDistanceError('CA', self.get_num(), residue.get_num())

    def _get_cb_distance(self, residue):
        '''
        Gets the distance between the CB of two residues (CA if Glycine)
        '''
        try:
            if self.is_gly() and residue.is_gly():
                return self.ca.get_distance(residue.ca)
            elif not self.is_gly() and residue.is_gly():
                return self.cb.get_distance(residue.ca)
            elif self.is_gly() and not residue.is_gly():
                return self.ca.get_distance(residue.cb)
            else:
                return self.cb.get_distance(residue.cb)
        except:
            raise ResidueDistanceError('CB', self.get_num(), residue.get_num())

    def get_residue_distance(self, residue, c_type):
        '''
        Gets the distance between two residues
        '''
        c_type_accepted = ('CA','CB','MIN')
        if c_type.upper() not in c_type_accepted:
            sys.stderr.write('Residue %d: Incorrect c_type: %s. Using CB...\n' % (self.get_num(), c_type))
        try:
            if c_type.upper() == 'CA':
                distance = self._get_ca_distance(residue)
            elif c_type.upper() == 'MIN':
                distance = self._get_min_distance(residue)
            else:
                distance = self._get_cb_distance(residue)
        except ResidueDistanceError as e:
            raise e
        return distance

    def relocate(self, translation_vector, rotation_vector, docking='HEX'):
        '''
        Relocate the residue position 
        @translation_vector = Vector used to translate the structure
        @Rotation_vector = Vector used to rotate the structure
        @docking = Docking program used to get the vectors
        '''
        for atom in self:
            try:
                atom.relocate(translation_vector, rotation_vector, docking)
            except RelocateProgramError as e:
                raise e

    def __iter__(self):
        '''
        Iterates over each atom
        '''
        for atom in self.atoms:
            yield atom

    def __str__(self):
        '''
        Return the information of the residue
        '''

        residueInfo = ''
        for atom in self:
            residueInfo += 'ATOM%s  %s%s%s%s%s%s\n' % (str(atom.get_num()).rjust(7),
                                                       atom.get_type().ljust(4),
                                                       self.get_type().rjust(3),
                                                       str(self.get_num()).rjust(4),
                                                       ('%.3f' % float(atom.get_x())).rjust(12),
                                                       ('%.3f' % float(atom.get_y())).rjust(8),
                                                       ('%.3f' % float(atom.get_z())).rjust(8))
        return residueInfo



import sys
import argparse
import os
import re
import ConfigParser
import shutil
import subprocess
import gzip

# Add '.' to sys.path
src_path = os.path.abspath(os.path.dirname(__file__))+"/../../scripts"
sys.path.append(src_path)

# Read configuration file
config = ConfigParser.ConfigParser()
config_file = os.path.join(src_path, 'config.ini')
config.read(config_file)

# Add SBI library path to sys.path
src_path = os.path.join(config.get('Paths','modppi_path'),config.get('Paths', 'sbi_library_path'))
sys.path.append(src_path)

# Add functions path to sys.path
src_path = os.path.join( config.get('Paths','modppi_path'), config.get('Paths', 'functions_path') )
sys.path.append(src_path)

# Import functions

from SBI.structure import PDB
import SBI.structure.contacts as cn
from functions import *
from SeqIO import *


def main():
   #Initialize
   options=parse_user_arguments()
   verbose=options.show
   pdb_path=os.path.join( config.get('Paths','modppi_path'),config.get('Paths', 'pdb_path') )
   dummy_dir=options.dummy_dir
   try:
    did_path=os.path.join(config.get('Paths','modppi_path'),config.get('Paths','3did_path'))
    data_path=os.path.join(config.get('Paths','modppi_path'),config.get('Paths','data_path'))
   except:
    did_path=options.outdir
    data_path=options.outdir

   if not os.path.exists(did_path):
    os.makedirs(did_path)
   if not os.path.exists(dummy_dir):
    os.makedirs(dummy_dir)
   if not os.path.exists(data_path):
    sys.stderr.write("No DATA directory, please check your installation or INPUT\n")

   #Parse did flat file
   did=parse_3did(options)

   #Create PDB files of 3DiD interactions
   for dd,cases in did.iteritems():
      for label in xrange(0,len(cases)):
        #Define the name of the PDB output file with domain-domain interactions
        did_file=os.path.join(did_path,dd[0]+":"+dd[1]+"#"+str(label)+".brk.gz")
        if not os.path.exists(did_file.lower()):
          did_file=os.path.join(did_path,dd[0]+":"+dd[1]+"#"+str(label)+".brk")
        if not os.path.exists(did_file.lower()):
          if verbose: sys.stderr.write("\t\t--Create %s\n"%(did_file.lower()))
          pdb_code,d1,d2 = cases[label]
          pdb_file=os.path.join(pdb_path,pdb_code[1:3].lower(),"pdb"+pdb_code+".ent")
          if not os.path.exists(pdb_file):
             pdb_file=os.path.join(pdb_path,pdb_code[1:3].lower(),"pdb"+pdb_code+".ent.gz")
          if not os.path.exists(pdb_file):
            if verbose: sys.stderr.write("\t\t\t-- %s not found\n"%pdb_file)
            continue
          try:
            pdb=PDB(pdb_file)
            brk=PDB()
            pdb_chain_A=pdb.get_chain_by_id(d1[0])
            start_A=d1[1]
            end_A  =d1[2]
            pdb_chain_B=pdb.get_chain_by_id(d2[0])
            start_B=d2[1]
            end_B  =d2[2]
            brk_chain_A=pdb_chain_A.extract(init=start_A,end=end_A)
            brk_chain_A.chain="A"
            brk.add_chain(brk_chain_A)
            brk_chain_B=pdb_chain_B.extract(init=start_B,end=end_B)
            brk_chain_B.chain="B"
            brk.add_chain(brk_chain_B)
            brk.clean()
            brk.write(did_file.lower())
          except Exception as e:
            if verbose: sys.stderr.write("\t\t\t  Error: %s\n"%e)
            continue
          

   #Create list of interactions and FASTA sequences of 3DiD
   did_interactions=open(os.path.join(data_path,options.interactions_file),"w")
   did_fasta       =open(os.path.join(data_path,options.seq_file),"w")
   for brk in os.listdir(did_path):
       if verbose: sys.stderr.write("\t\t-- Reading %s  \n"%os.path.join(did_path,brk))
       try:
          pdb=PDB(os.path.join(did_path,brk))
          id_chain=[]
          for c in pdb.chain_identifiers:
           pdb_chain=pdb.get_chain_by_id(c)
           id_chain.append(pdb.id+"_"+c)
           printfasta( did_fasta, pdb.id+"_"+c , pdb_chain.gapped_protein_sequence)      
          did_interactions.write("%s\t%s\n"%(id_chain[0],id_chain[1]))
       except Exception as e:
          if verbose: sys.stderr.write("\t\t-- %s cannot be read\n\t\t   Error: %s\n"%(os.path.join(did_path,brk),e))
          continue
   did_interactions.close()
   did_fasta.close()
          

def parse_user_arguments(*args, **kwds):
    parser = argparse.ArgumentParser(
        description = 'Generate the datase of DDI with the 3DiD flat file',
        epilog      = '@Oliva\'s lab 2016')
    parser.add_argument('-i', '--3did', dest = 'flat_file', action = 'store',
                        help = 'Input file: 3did_flat file ', default = '3did_flat')
    parser.add_argument('-ppi', '--interactions', dest = 'interactions_file', action = 'store',
                        help = 'Output file with a list of the interactions from 3did')
    parser.add_argument('-seq', '--fasta', dest = 'seq_file', action = 'store',
                        help = 'Output file with FASTA sequences of the proteins from 3DiD')
    parser.add_argument('-o', '--output_directory', dest = 'outdir', action = 'store', default = '3did',
                        help = 'Output directory (default is 3did)')
    parser.add_argument('-d', '--dummy_dir', dest = 'dummy_dir', action = 'store', default = '/tmp/3did_dummy',
                        help = 'Location of the dummy folder (default is /tmp/3did_dummy)')
    parser.add_argument('-v', '--verbose', dest = 'show', action = 'store_true',
                        help = 'Flag for verbose mode (default is False)')
    options = parser.parse_args()
    return options

def parse_3did(options):

 did={}
 flat=open(options.flat_file,'r')
 for line in flat:
   if line.startswith("#=ID"):
      word=line.split()
      pf1=word[1]
      pf2=word[2]
      ddi=(pf1,pf2)
   if line.startswith("#=3D"):
      word=line.split()
      pdb=word[1]
      frame1=word[2]
      frame2=word[3]
      chain1=frame1.split(":")[0]
      chain2=frame2.split(":")[0]
      section1=frame1.split(":")[1]
      section2=frame2.split(":")[1]
      start1=section1.split("-")[0]
      start2=section2.split("-")[0]
      end1=section1.split("-")[1]
      end2=section2.split("-")[1]
      d1=(chain1,start1,end1)
      d2=(chain2,start2,end2)
      # Stored information in did: PDB code, and two domain fragments. 
      # Each domain fragment is defined by Chain and Aas interval (start-end)
      triad=(pdb,d1,d2)
      did.setdefault(ddi,[]).append(triad)

 return did


if __name__ == '__main__':
    main()





import sys
import os
import inspect
import traceback

class parameters(object):
    """parameters controls some IO minor properties

        To use, include the SBIglobals variable in your script and modify.
    """
    def __init__(self):
        self._verbose   = False      # Promps extra information during different processes performed by the library
        self._debug     = False      # Promps even more information than verbose (use when fixing or modifying the library)
        self._ddebug    = False      # As debug, but for things that are going to repeat itsef a lot (like in Residue or Contact)
        self._overwrite = False      # General set to force overwrite existing files in all the processes.
                                     # Local overwrite (available in different functions) MUST take precedence
        self._warning   = False      # Promps advice if needed
        self._error     = True       # Explains errors
        self._fd        = sys.stderr # Where to print the stuff
    #
    # ATTRIBUTES
    #
    @property
    def verbose(self):          return self._verbose
    @verbose.setter
    def verbose(self, value):   self._verbose = value

    @property
    def debug(self):            return self._debug
    @debug.setter
    def debug(self, value):
        if value:  self._verbose, self._debug = value, value
        else:      self._debug                = value

    @property
    def deepdebug(self):        return self._debug
    @debug.setter
    def deepdebug(self, value):
        if value:  self._verbose, self._debug, self._ddebug = value, value, value
        else:      self._ddebug                             = value

    @property
    def warning(self):          return self.warning
    @warning.setter
    def warning(self, value):   self._warning = value

    @property
    def overwrite(self):        return self._overwrite
    @overwrite.setter
    def overwrite(self, value): self._overwrite = value

    #
    # METHODS
    #
    def output_file(self, debug_file):
        self._fd = open(debug_file,'w')

    def set_file(self):
        execname = ".".join([os.path.splitext(inspect.getfile(inspect.currentframe()))[0], os.getpid(), 'debug'])
        self._fd = open(execname,'w')

    def alert(self, level, source_object, message, error = None, killit = True):
        if self._active_level(level):
            name = getattr(source_object, '__class__', source_object)
            tag  = ''
            if level   == 'error':   tag = '[[E!]]'
            elif level == 'warning': tag = '[[W!]]'
            self._fd.write('{0}[{1}]: {2}\n'.format(tag, name, message))
            if error is not None:
                self._fd.write(traceback.format_exc())
            if level == 'error':
                self._fd.flush()
                if killit: sys.exit()

    def error(self, source_object, message, error = None, killit = True):
        self.alert('error', source_object, message, error, killit)

    def decide_overwrite(self, local_overwrite):
        return self.overwrite if local_overwrite is None else local_overwrite
    #
    # PRIVATE METHODS
    #
    def _active_level(self, level):
        return (level == 'error'                         or (level == 'warning' and self.warning) or \
               (level == 'verbose'   and self.verbose)   or (level == 'debug'   and self.debug)   or \
               (level == 'deepdebug' and self.deepdebug))

SBIglobals = parameters()



"""GOlink

author: jbonet
date:   10/2013

@oliva's lab
"""

"""
Import Standard Libraries
"""
import os, re
import urllib

"""
Dependences in SBI library
"""
from SBI.databases import GOftp, GOnamespace
from SBI.beans.Path     import Path
from SBI.beans.File     import File

class GOlink(object):
    """The GOlink class controls the download and parsing of GO database

    """
    def __init__(self, local = None):
        self._local   = os.path.abspath(local)
        self._gfile   = 'go-simple.obo'
        self.__name__ = 'databases.GOlink'    # This must be included in every class for the SBIglobals.alert()
        self._gofile  = 'go.gz'
        if local is not None:
            self.local = local

    """ATTRIBUTES"""
    @property
    def local(self):         return self._local
    @local.setter
    def local(self, value):
        self._local  = os.path.abspath(value)
        self._gofile = os.path.join(self._local, self._gofile)

    @property
    def localGOs(self):
        goFile = File(self._gofile, 'r')
        for go_line in goFile.descriptor:
            yield go_line

    @property
    def source(self):
        return GOftp['show']

    """BOOLEANS"""
    @property
    def has_local(self):    return self._local is not None

    """METHODS"""
    def download(self):
        if not self.has_local:
            raise NameError('A local GO database directory must be defined.')

        Path.mkdir(self.local)
        destination = os.path.join(self.local, self._gfile)

        urllib.urlretrieve(GOftp['source'], destination)
        self._process()

        return True

    def get_GO(self, GO):
        if self.has_local:
            for go_line in self.localGOs:
                if go_line.split('\t')[0] == GO.upper():
                    return go_line
        else:
            raise NameError('A local GO database directory must be defined.')

    def get_GOs(self, GOset):
        if isintance(GOset, str):
            warnings.warn('For single GO search the get_GO function is recomended.')
            yield self.get_GO(GOset)
        else:
            GOset = set([x.upper() for x in GOset])

        if self.has_local:
            for go_line in self.localGOs:
                if go_line.split('\t')[0] in GOset:
                    yield go_line
        else:
            raise NameError('A local GO database directory must be defined.')

    """PRIVATE METHODS"""
    def _process(self):
        go_dic = {}
        parseFile = File(os.path.join(self.local, self._gfile), 'r')
        go = None
        for line in parseFile.descriptor:
            line = re.sub('\'', '\\\'', line)
            if line.startswith('[Term]'):
                if go is not None:
                    go_dic[go.id] = go
            if line.startswith('id:'):
                go = GOterm(id = line.split()[1].strip())
                continue
            if line.startswith('name:'):
                go.name = " ".join(line.split()[1:]).strip()
                continue
            if line.startswith('namespace:'):
                go.namespace = line.split()[1].strip()
                continue
            if line.startswith('alt_id:'):
                go.alt_id.append(line.split()[1].strip())
                continue
            if line.startswith('is_obsolete:'):
                go.obsolete = True
                continue
            if line.startswith('is_a:'):
                go.parents.add(line.split()[1].strip())
                continue
            if line.startswith('relationship:'):
                go.relations.append((line.split()[1].strip(),line.split()[2].strip()))
                continue
            if line.startswith('[Typedef]'):
                go_dic[go.id] = go
                break
        parseFile.close()

        for go in go_dic:
            go_dic[go].parents = self._search_parents(go_dic, go)

        goFile = File(self._gofile, 'w', True)
        for go in go_dic:
            go_dic[go].parents.add(go)
            goFile.write(str(go_dic[go]) + "\n")
        goFile.close()

    def _search_parents(self, godict, go):
        parents = set()
        for parent in godict[go].parents:
            parents.add(parent)
            parents.update(self._search_parents(godict, parent))
        return parents

class GOterm(object):

    def __init__(self, id = None, inline = None):
        if inline is not None:
            inline      = inline.strip()
        self.id         = id    if inline is None else inline.split('\t')[0]
        self.name       = None  if inline is None else inline.split('\t')[1]
        self._namespace = None  if inline is None else inline.split('\t')[2]
        self.obsolete   = False if inline is None else False if int(inline.split('\t')[3]) == 0 else True
        self.alt_id     = []    if inline is None else eval(inline.split('\t')[4])
        self.parents    = set() if inline is None else eval(inline.split('\t')[5])
        self.relations  = []    if inline is None else eval(inline.split('\t')[6])

    """ATTRIBUTES"""
    @property
    def namespace(self):        return self._namespace
    @namespace.setter
    def namespace(self, value): self._namespace = value
    @property
    def namespace_mini(self):   return GOnamespace[self._namespace]
    @property
    def obsolete_binary(self):  return 1 if self.obsolete else 0

    """OVERWRITE PARENT METHODS"""
    def __str__(self):
        return "{0.id}\t{0.name}\t{0.namespace}\t{0.obsolete_binary}\t{0.alt_id}\t{0.parents}\t{0.relations}".format(self)




'''
@file: IndexedNum.py

@author: Jaume Bonet
@mail:   jaume.bonet@gmail.com
@date:   2013

@ [oliva's lab](http://sbi.imim.es)

@class: IndexedNum
'''
import re


class IndexedNum(object):
    '''
    Created for PDB.
    Some PDB files contain residues in positions that are not only numeric,
    but also have an index.
    {IndexedNum} tries to palliate this by giving the properties of a number
    (as much as possible) to this kind of number-index combination.
    Thus, a 7 becomes a '7 ' but can still be used as a number.
    Comparisons, addition (to integers or fully integer {IndexedNum}) and int(),
    float() casts are implemented. It is hash-able and, thus, can be used as key
    in dictionaries.

    If it is initialized with X, it means that we have a "blank" position.
    To avoid mix-ups, the index is transformed to '_X_'
    '''
    _NUMBER_REGEX = re.compile('(\-*\d+)(\w*)')
    _REVERS_REGEX = re.compile('(\w{1})(\d*)')

    def __init__(self, value):
        '''
        @param:    value
        @pdef:     value to transform to a {IndexedNum}
        @ptype:    {Integer}, {String}, {IndexedNum}
        '''
        if isinstance(value, IndexedNum):
            self._num = value._num
            self._idx = value._idx
        else:
            value = str(value)
            if not value.startswith('X'):
                parts = re.search(self._NUMBER_REGEX, value)
                self._num = int(parts.group(1))
                self._idx = parts.group(2)
                if self._idx == '':
                    self._idx = ' '
            else:
                parts = re.search(self._REVERS_REGEX, value)
                if parts:
                    self._num = int(parts.group(2))
                else:
                    self._num = None
                self._idx = '_X_'

    ##############
    # ATTRIBUTES #
    ##############
    @property
    def number(self):
        '''
        Integer part value.

        @return: {Integer}
        '''
        return self._num

    @property
    def index(self):
        '''
        String part value.

        @return: {String}
        '''
        return self._idx

    ############
    # BOOLEANS #
    ############
    @property
    def is_integer(self):
        '''
        Is the value a pure Integer?

        @return: {Boolean}
        '''
        return self._idx == ' '

    @property
    def is_blank(self):
        '''
        Does it have no value?

        @return: {Boolean}
        '''
        return self._idx == '_X_'

    #################
    # CLASS METHODS #
    #################
    def __lt__(self, other):
        if isinstance(other, int):
            return self.number < other
        elif isinstance(other, IndexedNum):
            if self.number < other.number:
                return True
            elif self.number == other.number:
                return self.index < other.index
            else:
                return False
        return NotImplemented

    def __le__(self, other):
        if isinstance(other, int):
            return self.number <= other
        elif isinstance(other, IndexedNum):
            if self.number < other.number:
                return True
            elif self.number == other.number:
                return self.index <= other.index
            else:
                return False
        return NotImplemented

    def __eq__(self, other):
        if isinstance(other, int):
            return self.number == other
        elif isinstance(other, IndexedNum):
            return self.number == other.number and self.index == other.index
        elif isinstance(other, basestring):
            return str(self).strip() == other.strip()
        return NotImplemented

    def __ne__(self, other):
        result = self.__eq__(other)
        if result is NotImplemented:
            return result
        return not result

    def __gt__(self, other):
        if isinstance(other, int):
            return self.number > other
        elif isinstance(other, IndexedNum):
            if self.number > other.number:
                return True
            elif self.number == other.number:
                return self.index > other.index
            else:
                return False
        return NotImplemented

    def __ge__(self, other):
        if isinstance(other, int):
            return self.number >= other
        elif isinstance(other, IndexedNum):
            if self.number > other.number:
                return True
            elif self.number == other.number:
                return self.index >= other.index
            else:
                return False
        return NotImplemented

    def __add__(self, other):
        if isinstance(other, int):
            return IndexedNum(str(self.number + other) + self.index)
        if isinstance(other, IndexedNum) and other.is_integer:
            return IndexedNum(str(self.number + other) + self.index)
        return NotImplemented

    def __iadd__(self, other):
        if isinstance(other, int):
            self.number += other
        if isinstance(other, IndexedNum) and other.is_integer:
            self.number += other
        return NotImplemented

    def __int__(self):
        return int(self.number)

    def __float__(self):
        return float(self.number)

    def __hash__(self):
        return repr(self)

    def __repr__(self):
        if self.is_blank:
            if self._num is None:
                return 'X'
            else:
                return 'X' + str(self._num)
        else:
            return (str(self.number) + self.index).strip()

    def __str__(self):
        return repr(self)



"""
Nucleotide

author: jbonet
date:   02/2013

@oliva's lab
"""

from .          import Residue
from SBI.data   import nucleic2to1
import numpy    as np

class ResidueOfNucleotide(Residue):
    """
    A {Nucleotide} collects a series of {NucleotideAtom}s
    """
    def __init__(self, number, version, Rtype, mode):
	"""
        @type  number: Integer
        @param number: Residue number

        @type  version: Char
        @param version: Optional char used on pdbs to change count

        @type  type: String
        @param type: Residue type

        @type  mode: String
        @param mode: Residue mode: ATOM or HETATM
        """
        super(ResidueOfNucleotide, self).__init__(number = number, version = version, Rtype = Rtype, mode = mode)
        

    #
    # ATTRIBUTES
    #
    @property
    def single_letter(self):
        """
        Returns the AminoAcid identifier as a single letter code
        @rtype: String
        """
        return nucleic2to1[self.type]



from SBI.databases import GOterm

tables = {'go'     : 'GO', 
          'GO_alt' : 'GO_alternative',
          'GO_ext' : 'GO_extended',
          'GO_rel' : 'GO_relationship'}

class GOterm(GOterm):
    def __init__(self, id = None, inline = None):
        super(GOterm, self).__init__(id = id, inline = inline)

    """"""
    def toSQL(self):
        data = []
        data.append('INSERT INTO {0} (id, name, namespace, obsolete) VALUES ("{1.id}","{1.name}","{1.namespace_mini}",{1.obsolete_binary});'.format(tables['go'], self))
        data.append('SET @goid = (SELECT nid FROM {0} WHERE id="{1.id}");'.format(tables['go'], self))
        for alt in self.alt_id:
            data.append('INSERT INTO {0} VALUES (@goid, "{1}");'.format(tables['GO_alt'], alt))
        return "\n".join(data)

    def relations2SQL(self):
        data = []
        data.append('SET @goid1 = (SELECT nid FROM {0} WHERE id="{1.id}");'.format(tables['go'], self))
        for rel in self.relations:
            data.append('SET @goid2 = (SELECT nid FROM {0} WHERE id="{1}");'.format(tables['go'], rel[1]))
            data.append('INSERT INTO {0} VALUES (@goid1, @goid2, "{1}");'.format(tables['GO_rel'], rel[0]))
        return "\n".join(data)

    def parents2SQL(self):
        data = []
        data.append('SET @goid1 = (SELECT nid FROM {0} WHERE id="{1.id}");'.format(tables['go'], self))
        for rel in self.parents:
            data.append('SET @goid2 = (SELECT nid FROM {0} WHERE id="{1}");'.format(tables['go'], rel))
            data.append('INSERT INTO {0} VALUES (@goid2, @goid1);'.format(tables['GO_ext']))
        return "\n".join(data)